      PROGRAM FIFE
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           FIFE            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     PROGRAM :
C*          FRIENDLY INTERACTIVE FORTRAN ENVIRONMENT
C*
C*     AUTHOR :
C*          ART RAGOSTA   &   LAURA JURGELEIT
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA.   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          THIS PROGRAM IS THE GENERAL-PURPOSE EXECUTIVE THAT
C*          IMPLEMENTS THE ARTA COMMAND INTERFACE FOR FORTRAN SUBPROGRAMS
C*
C*     COMMON BLOCKS :
C*          SYSTEM
C*
C*     SUBPROGRAM REFERENCES :
C*          INITIAL,  GET_COMMAND,  DO_COMMAND,  ENDUP,  EXIT
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0
C*
C*     CHANGE HISTORY :
C*          NOVEMBER 23, 1987  -  INTIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
C
      CALL INITIAL
C
C --- LOOP UNTIL 'QUIT' IS ENTERED OR A FATAL ERROR OCCURS
C
100   CALL GET_COMMAND
      CALL DO_COMMAND
      IF (.NOT. DONE) GO TO 100
C
      CALL ENDUP
      END
C
C---END FIFE
C
      SUBROUTINE INITIAL
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          INITIAL          **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          INITIALIZE SYSTEM-WIDE STUFF
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA.   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          INITIALIZE PROGRAM-WIDE POINTERS AND SET DEFAULT VALUES.
C*
C*     INTERNAL WORK AREAS :
C*          P  - PARAMETERS FROM THE INVOCATION LINE
C*          Q  - QUALIFIERS FROM THE INVOCATION LINE
C*
C*     COMMON BLOCKS :
C*          SYSTEM, STACK, MACRO
C*
C*     FILE REFERENCES :
C*          NREAD - INITIALIZATION FILE READ (IF ANY)
C*
C*     SUBPROGRAM REFERENCES :
C*          GETFOR
C*
C*     ERROR PROCESSING :
C*          CHECK FOR INPUT FILE EXISTENCE
C*
C*     TRANSPORTABILITY LIMITATIONS :
C*          NONE
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          REQUIRES 'MERLIB'
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0        20 NOV 1987
C*
C*     CHANGE HISTORY :
C*          20-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      INCLUDE 'FIFE_STACK.CMN'
      INCLUDE 'FIFE_MACRO.CMN'
      INCLUDE 'FIFE_COMMANDS.CMN'
      INCLUDE 'FIFE_UNITS.CMN'
      CHARACTER *40 Q(5), P(5)
      EXTERNAL EDIT, ENDIT, HARDBALL, HELPER, INCLUDER, MACRO, SPAWN
      EXTERNAL DISPLAY
C
      DONE   = .FALSE.
      WRITE (NWRITE,900) VERSION
C
C --- INITIALIZE DEFAULT VALUES
C
      NSTACK = 0
      NMACRO = 0
      MPTR(1)= 1
C
C --- GET PARAMETERS AND QUALIFIERS
C
      CALL GETFOR ( NQ, Q, NP, P )
C
C --- INITIALIZE COMMAND LIST ADDRESS ARRAY
C
      COMMANDS(1) = '?         '
      LIST(1) = %LOC(HELPER)
      COMMANDS(2) = 'HELP      '
      LIST(2) = %LOC(HELPER)
C
      COMMANDS(3) = 'ADAM      '
      LIST(3) = %LOC(EDIT)
      COMMANDS(4) = 'EDIT      '
      LIST(4) = %LOC(EDIT)
C
      COMMANDS(5) = 'DCL       '
      LIST(5) = %LOC(SPAWN)
      COMMANDS(6) = 'VMS       '
      LIST(6) = %LOC(SPAWN)
      COMMANDS(7) = 'SPAWN     '
      LIST(7) = %LOC(SPAWN)
C
      COMMANDS(8) = 'DISPLAY   '
      LIST(8) = %LOC(DISPLAY)
      COMMANDS(9) = 'HARDBALL  '
      LIST(9) = %LOC(HARDBALL)
C
      COMMANDS(10) = 'END       '
      LIST(10) = %LOC(ENDIT)
      COMMANDS(11) = 'QUIT      '
      LIST(11) = %LOC(ENDIT)
      COMMANDS(12) = 'TERMINATE '
      LIST(12) = %LOC(ENDIT)
      COMMANDS(13) = 'EXIT      '
      LIST(13) = %LOC(ENDIT)
C
      COMMANDS(14) = 'READ      '
      LIST(14) = %LOC(INCLUDER)
      COMMANDS(15) = 'INCLUDE   '
      LIST(15) = %LOC(INCLUDER)
C
      COMMANDS(16) = 'MACRO     '
      LIST(16) = %LOC(MACRO)
      NCOMMANDS = 16
C
C --- SORT THE COMMAND ARRAY
C
      CALL SORT2 ( COMMANDS, LIST, NCOMMANDS )
C
C --- SET END-OF-FILE ON READ FLAG TO POINT TO "END"
C
      DO 10 I = 1, NCOMMANDS
         IF (COMMANDS(I) .EQ. 'END') THEN
            N_EOF = I
            GO TO 15
         ENDIF 
10    CONTINUE
C
C --- CHECK FOR INITIALIZATION FILE IN SYS$LOGIN
C
15    NREAD = NREAD + 2
      OPEN (UNIT=NREAD, NAME=INIT_FILE, STATUS='OLD', ERR=20)
      RETURN
20    NREAD = NREAD - 2
      RETURN
C
900   FORMAT(////,
     $ T20,'---*****   FIFE   *****---',//,
     $ '      ',A,//////,
     $ ' Enter "HELP" at the FIFE> prompt to get a general description'/
     $ ' of the program and a list of commands.  Enter  "HELP command"'/
     $ ' to get help on a specific command.  Enter "command ?" for a'/
     $ ' list of command options (if applicable).'/)
      END
C
C---END INITIAL
C
      SUBROUTINE SORT2 ( ARRAY, ARRAY1, NUM )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           SORT            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          SORT ARRAY - THE INPUT ARRAYS ARE SORTED USING A SHELL SORT.
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 207-5                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF 94035              
C*          (415)604-5578                           
C*
C*     PURPOSE :
C*          REPLACE AN ARRAY WITH A SORTED ARRAY.
C*
C*     METHODOLOGY :
C*          SHELLSORT                        
C*
C*     INPUT ARGUMENTS :
C*          ARRAY  - ARRAY TO BE SORTED
C*          NUM    - NUMBER OF ELEMENTS IN ARRAY
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          THE TYPE OF THE ARRAY 'ARRAY' AND THE VARIABLE 'TEMPA'
C*          MUST BE SET FOR EACH TYPE OF SORT. THE SAMPLE BELOW IS FOR
C*          CHARACTER*255 (OR BELOW) VARIABLES.
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0     MARCH 12, 1984
C*
C*     CHANGE HISTORY :
C*          03/12/84     INITIAL VERSION
C*
C***********************************************************************
C*
      DIMENSION ARRAY(1), ARRAY1(1)
      CHARACTER *20 ARRAY
      INTEGER ARRAY1
      CHARACTER *20 TEMPA
      LOGICAL DONE
C
      IF (NUM .LE. 1) RETURN
      JUMP = NUM
20    JUMP = JUMP / 2
30    DONE = .TRUE.
      NJ = NUM-JUMP
      DO 40 J = 1, NJ
         I = J + JUMP
         IF (ARRAY(J) .GT. ARRAY(I))THEN
            DONE = .FALSE.
            TEMPA = ARRAY(J)
            ITEMP = ARRAY1(J)
            ARRAY(J) = ARRAY(I)
            ARRAY1(J) = ARRAY1(I)
            ARRAY(I) = TEMPA
            ARRAY1(I) = ITEMP
         ENDIF
40       CONTINUE
      IF (.NOT. DONE) GO TO 30
      IF (JUMP .GT. 1) GO TO 20
      RETURN
      END
C
C---END SORT2
C
      SUBROUTINE ENDUP
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          ENDUP            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          END UP
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          CLEANUP IN ANTICIPATION OF EXITING PROGRAM.
C*
C*     SUBPROGRAMS CALLED :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     TRANSPORTABILITY LIMITATIONS :
C*          NONE
C*
C*     VERSION AND DATE :
C*          VERSION I.0      1-NOV-87
C*
C*     CHANGE HISTORY :
C*           1-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      CALL EXIT
      END
C
C---END ENDUP
C
      SUBROUTINE GET_COMMAND
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **       GET_COMMAND         **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET COMMAND
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA.   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          READ THE NEXT COMMAND FROM THE PRESENT COMMAND FILE.  PARSE
C*          IT INTO TOKEN FOR THE INTERPRETATION STEP.
C*
C*     COMMON BLOCKS :
C*          SYSTEM, STACK, UNITS
C*
C*     FILE REFERENCES :
C*          NREAD - PASSED TO READLN; DECREMENTED IF AN EOF IS ENCOUNTERED
C*
C*     SUBPROGRAM REFERENCES :
C*          READLN,  CAPS,  GETOKE,  ERROR
C*
C*     ERROR PROCESSING :
C*          CHECK ERRORS PASSED BY GETOKE AND READLN
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          REQUIRES 'MERLIB'
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      20-NOV-87
C*
C*     CHANGE HISTORY :
C*           20-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      INCLUDE 'FIFE_UNITS.CMN'
      INCLUDE 'FIFE_COMMANDS.CMN'
      CHARACTER *1 TYPE
      LOGICAL ERR
C
C --- THIS IS THE ONLY COMMAND READ IN THE PROGRAM!
C
10    CALL READLN ( LL, IERR )
      IF ( IERR .EQ. 2 ) THEN
         CALL ERROR(2,'Error reading input line or line overflow.')
         GO TO 10
      ENDIF
      IF ( IERR .EQ. 1 ) THEN
C
C --- END OF FILE REACHED...
C ---  IF WE ARE IN A FILE (NREAD > 5) RETURN TO CALLING UNIT
C ---  OTHERWISE EXIT PROGRAM
C
         IF (NREAD .EQ. 5) THEN
            IF (N_EOF .NE. 0) TOKEN(1) = COMMANDS ( N_EOF )
            NTOKE = 1
            RETURN
         ELSE
            CLOSE(UNIT=NREAD)
            NREAD = NREAD - 2
            GO TO 10
         ENDIF
      ENDIF
C
C --- IGNORE BLANK LINES
C
      IF ( LL .EQ. 0 ) GO TO 10
C
C --- LINE READ WITH NO ERRORS
C
C --- 'SLINE' IS LEFT UNCAPITALIZED FOR TEXTUAL INPUT
C
      SLINE = LINE
      CALL CAPS ( LINE )
      IP    = 0
      NTOKE = 0
C
C --- PARSE LINE
C
100   CALL GETOKE ( LINE, LL, IP, TOKEN(NTOKE+1), TYPE, ERR )
      IF ( ERR ) THEN
         CALL ERROR (2,'Illegal symbol in line.')
      ELSE
         IF (TYPE .NE. 'E') THEN
            NTOKE = NTOKE + 1
            IF ( NTOKE .LT. MAX_TOKE) GO TO 100
            CALL ERROR (2,'Too many TOKENS in line.')
         ENDIF
      ENDIF
      RETURN
      END
C
C---END GET_COMMAND
C
      SUBROUTINE DO_COMMAND
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **        DO_COMMAND         **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DO COMMAND
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA.   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          THIS ROUTINE DETERMINES THAT THE COMMAND LINE STARTS WITH
C*          A VALID COMMAND KEYWORD AND CONTROLS THE EXECUTION OF THE
C*          COMMAND IF IT IS VALID.
C*
C*     COMMON BLOCKS :
C*          SYSTEM, COMMANDS, STACK, MACRO
C*
C*     SUBPROGRAM REFERENCES :
C*          SEARCH,  ERROR,  DOIT
C*
C*     ERROR PROCESSING :
C*          CHECK ERRORS SENT FROM SEARCH
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          REQUIRES 'MERLIB'
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      20-NOV-87
C*
C*     CHANGE HISTORY :
C*           20-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      INCLUDE 'FIFE_STACK.CMN'
      INCLUDE 'FIFE_COMMANDS.CMN'
      INCLUDE 'FIFE_MACRO.CMN'
      CHARACTER *80 TEMP
      CHARACTER *20 WORK
      LOGICAL AMBIG, MATCH
C
C --- SEARCH LIST OF KNOWN COMMANDS FOR A NON-AMBIGUOUS MATCH
C
      CALL SEARCH (COMMANDS, NCOMMANDS, TOKEN(1), K, MATCH, AMBIG)
      IF ( .NOT. MATCH ) THEN
C
C --- IT'S NOT A SYSTEM COMMAND, PERHAPS A MACRO?
C
         DO 10 I=1,NMACRO
            IF (TOKEN(1) .EQ. MNAME(I)) GO TO 20
10          CONTINUE
C
C --- GUESS NOT
C
         CALL ERROR(2,'Unknown command '//TOKEN(1) )
      ELSE IF ( AMBIG ) THEN
         CALL ERROR
     $   (2,'Ambiguous command verb... use more characters.')
      ELSE
C
C --- COMMAND WAS FOUND
C
         CALL DOIT ( %VAL(LIST(K)) )
      ENDIF
      RETURN
C
C --- MACRO EXPANSION
C
20    MMEND = MPTR(I+1) - 1
      DO 30 J = MMEND, MPTR(I), -1
         IF (NSTACK .EQ. MAXSTACK) CALL ERROR(3,'Macro Stack overflow.')
         NSTACK = NSTACK + 1
         TEMP = MTEXT(J)
C
C --- REPLACE PARAMS, IF ANY ($1 - $9)
C
25       IA = INDEX(TEMP,'$')
         IF (IA .NE. 0) THEN
            NUM = ICHAR(TEMP(IA+1:IA+1)) - ICHAR('0') + 1
            IF (( NUM .LT. 1) .OR. (NUM .GT. 9)) THEN
               CALL ERROR(2,'DUMMY ARGUMENTS ARE LIMITED TO THE '//
     $            'RANGE $1 - $9.')
               NUM = 1
            ENDIF
            IF (NUM .GT. NTOKE) THEN
               WORK = ' '
            ELSE
               WORK = TOKEN(NUM)
            ENDIF
            TEMP = TEMP(1:IA-1) // WORK(1:MAX(1,LENGTH(WORK))) //
     $              TEMP(IA+2:)
            GO TO 25
         ENDIF
         LSTACK(NSTACK) = TEMP
30       CONTINUE
      RETURN
      END
C
C---END DO_COMMAND
C
      SUBROUTINE GETYES ( FLAG )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETYES           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET YES
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          GET THE RESPONSE FOR A BOOLEAN OPTION.
C*
C*     OUTPUT ARGUMENTS :
C*          FLAG - THE BOOLEAN VALUE RETRIEVED
C*
C*     COMMON BLOCKS :
C*          SYSTEM, UNITS
C*
C*     SUBPROGRAM REFERENCES :
C*          ERROR
C*
C*     ERROR PROCESSING :
C*          CHECK TO MAKE SURE THERE IS A VALUE AND IT IS VALID
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      20-NOV-87
C*
C*     CHANGE HISTORY :
C*           20-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      INCLUDE 'FIFE_UNITS.CMN'
      LOGICAL FLAG
C
      IF (NTOKE .GT. 2) CALL ERROR(1,'Extra entries on line ignored.')
      IF (NTOKE .LT. 2) THEN
         FLAG = .FALSE.
         RETURN
      ENDIF
C
      IF (TOKEN(2) .EQ. '?') THEN
         WRITE(NWRITE,900)
         RETURN
      ENDIF
C
      IF ((TOKEN(2) .EQ. 'ON') .OR. (TOKEN(2) .EQ. '1') .OR.
     $    (TOKEN(2)(1:1) .EQ. 'Y') .OR. (TOKEN(2)(1:1) .EQ. 'T')) THEN
         FLAG = .TRUE.
      ELSE IF ((TOKEN(2) .EQ. 'OFF') .OR. (TOKEN(2) .EQ. '0') .OR.
     $    (TOKEN(2)(1:1) .EQ. 'N') .OR. (TOKEN(2)(1:1) .EQ. 'F')) THEN
         FLAG = .FALSE.
      ELSE
         CALL ERROR ( 2,'Boolean value expected.')
      ENDIF
      RETURN
900   FORMAT(' The valid choices for a boolean response are :',/,
     $       '     TRUE       FALSE',/,
     $       '     YES        NO',/,
     $       '     ON         OFF',/,
     $       '     1          0',//,
     $       ' All choices may be abbreviated to 1 or 2 non-ambiguous ',
     $       'characters.')
      END
C
C---END GETYES
C
      SUBROUTINE GETFIL ( NSTART, NAME, FTYPE )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETFIL           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET FILE NAME
C*
C*     AUTHOR :
C*          Art Ragosta
C*          MS 207-5
C*          Ames Research Center
C*          Moffett Field, Ca  94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          TO EXTRACT THE NAME FOR A FILE FROM THE COMMAND LINE, WITH
C*          A DEFAULT FILE TYPE
C*
C*     INPUT ARGUMENTS :
C*          NSTART - THE FIRST TOKEN IN THE LINE TO CHECK
C*          FTYPE  - DEFAULT FILE TYPE
C*
C*     OUTPUT ARGUMENTS :
C*          NAME   - THE FULL NAME FOR THE FILE
C*
C*     COMMON BLOCKS :
C*          SYSTEM
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH,  ERROR
C*
C*     ERROR PROCESSING :
C*          MAKE SURE A NAME IS ENTERED
C*
C*     TRANSPORTABILITY LIMITATIONS :
C*          THE VAX FILE SPECIFICATION IS ASSUMED (IE [DIR]NAME.TYP )
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0     20-NOV-1987
C*
C*     CHANGE HISTORY :
C*          20-NOV-87       INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      CHARACTER *(*) FTYPE
      CHARACTER *(*) NAME
C
      IF (NTOKE .LT. NSTART) THEN
         CALL ERROR(2,'File specification required.')
      ELSE
C
C ---- PACK THE STRING BACK TOGETHER
C
         NAME = ' '
         IS = 1
         DO 10 I = NSTART,NTOKE
            LT = LENGTH(TOKEN(I))
            NAME(IS:IS+LT-1) = TOKEN(I)(1:LT)
            IS = IS + LT
10          CONTINUE
C
C --- CHECK FOR FILE TYPE EXTENSION
C
         DO 20 I = IS,1,-1
            IF (NAME(I:I) .EQ. '.') THEN
               IF (I .EQ. (IS-1)) NAME(IS:)=FTYPE
               RETURN
            ENDIF
            IF (NAME(I:I) .EQ. ']') GO TO 25
20          CONTINUE
25       NAME(IS:) = '.' // FTYPE
      ENDIF
      RETURN
      END
C
C---END GETFIL
C
      SUBROUTINE GETEXT ( TEX )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETEXT           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET TEXT
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          GET THE VALUE OF A TEXT STRING FROM THE INPUT LINE.
C*
C*     OUTPUT ARGUMENTS :
C*          TEX - THE TEXT RETRIEVED
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH,  ERROR,  LEFT
C*
C*     ERROR PROCESSING :
C*          CHECK TO MAKE SURE THERE IS AN ENTRY
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          REQUIRES 'MERLIB'
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      20-NOV-87
C*
C*     CHANGE HISTORY :
C*           20-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      CHARACTER *(*) TEX
C
      IF (NTOKE .LT. 2) THEN
         CALL ERROR(1,'No value entered for textual item.')
         TEX = ' '
         RETURN
      ENDIF
C
C --- SKIP SPACES
C
      LTEX = LEN(TEX)
      CALL LEFT ( SLINE )
C
C --- SKIP KEYWORD
C
      IS = 0
10    IS = IS + 1
      IF (SLINE(IS:IS) .NE. ' ') GO TO 10
C
C --- SKIP SPACES AND QUOTES (IF ANY)
C
      IE = LENGTH ( SLINE )
20    IS = IS + 1
      IF ((SLINE(IS:IS) .EQ. ' ') .AND. (IS .LT. 80)) GO TO 20
      IF ((SLINE(IS:IS) .EQ. '"') .OR. (SLINE(IS:IS) .EQ. ''''))
     $    IS = IS + 1
      IF ((SLINE(IE:IE) .EQ. '"') .OR. (SLINE(IE:IE) .EQ. ''''))
     $    IE = IE - 1
C
C --- IF THERE ARE STILL CHARACTERS LEFT, ADD '$' TO END OF STRING FOR DISSPLA
C
      IF (IS .GT. IE) THEN
         CALL ERROR(2,'Text string expected.')
      ELSE
         IF ((IE+2-IS) .GT. LTEX) THEN
            CALL ERROR(1,'Text string truncated.')
            IE  = LTEX + IS - 2
         ENDIF
         TEX = SLINE(IS:IE)
         LT = LENGTH(TEX)
         IF (LT .GE. LTEX) THEN
            CALL ERROR(1,'Text string truncated.')
            LT = LTEX-1
         ENDIF
      ENDIF
      RETURN
      END
C
C---END GETEXT
C
      SUBROUTINE GETEXT1 ( TEX )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETEXT           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET TEXT (SPECIAL VERSION FOR TEXT COMMAND)
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          GET THE VALUE OF A TEXT STRING FROM THE INPUT LINE.
C*
C*     OUTPUT ARGUMENTS :
C*          TEX - THE TEXT RETRIEVED
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH,  ERROR,  LEFT
C*
C*     ERROR PROCESSING :
C*          CHECK TO MAKE SURE THERE IS AN ENTRY
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          REQUIRES 'MERLIB'
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      20-NOV-87
C*
C*     CHANGE HISTORY :
C*          20-NOV-87        INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      CHARACTER *(*) TEX
C
      IF (NTOKE .LT. 2) THEN
         CALL ERROR(1,'No value entered for textual item.')
         TEX = ' '
         RETURN
      ENDIF
C
C --- SKIP SPACES
C
      LTEX = LEN(TEX)
      CALL LEFT ( SLINE )
C
C --- SKIP KEYWORD AND X,Y
C
      ISS = 0
      IS = 0
10    IS = IS + 1
      IF (SLINE(IS:IS) .NE. ' ') GO TO 10
      IF (ISS .LT. 2) THEN
15       IS = IS + 1
         IF (SLINE(IS:IS) .EQ. ' ') GO TO 15
         ISS = ISS + 1
         GO TO 10
      ENDIF
C
C --- SKIP SPACES AND QUOTES (IF ANY)
C
      IE = LENGTH ( SLINE )
20    IS = IS + 1
      IF ((SLINE(IS:IS) .EQ. ' ') .AND. (IS .LT. 80)) GO TO 20
      IF ((SLINE(IS:IS) .EQ. '"') .OR. (SLINE(IS:IS) .EQ. ''''))
     $    IS = IS + 1
      IF ((SLINE(IE:IE) .EQ. '"') .OR. (SLINE(IE:IE) .EQ. ''''))
     $    IE = IE - 1
C
      IF (IS .GT. IE) THEN
         CALL ERROR(2,'Text string expected.')
      ELSE
         IF ((IE+2-IS) .GT. LTEX) THEN
            CALL ERROR(1,'Text string truncated.')
            IE  = LTEX + IS - 2
         ENDIF
         TEX = SLINE(IS:IE)
         LT = LENGTH(TEX)
         IF (LT .GE. LTEX) THEN
            CALL ERROR(1,'Text string truncated.')
            LT = LTEX-1
         ENDIF
      ENDIF
      RETURN
      END
C
C---END GETEXT1
C
      SUBROUTINE READLN ( LN, IERR )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **        READ LINE          **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          READ LINE
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS 207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA  94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          READ ONE OR MORE LINES OF INPUT, CAPITALIZE, DELETE COMMENTS
C*          AND CONTINUE READING IF CONTINUATION SPECIFIED (...).
C*
C*     INPUT ARGUMENTS :
C*          NONE
C*
C*     OUTPUT ARGUMENTS :
C*          IERR = 0 FOR NO ERROR
C*               = 1 FOR EOF
C*               = 2 FOR ERROR
C*          LN    - NUMBER OF CHARACTERS RETURNED IN LINE.
C*
C*     INTERNAL WORK AREAS :
C*          STRING - 80 CHARACTER BUFFER FOR READS FROM TERMINAL.
C*
C*     FILE REFERENCES :
C*          NREAD
C*
C*     SUBPROGRAM REFERENCES :
C*          PROMPT,  ERROR
C*
C*     ERROR PROCESSING :
C*          THE LINE LENGTH IS NOT ALLOWED TO EXCEED LEN(LINE)
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      20-NOV-1987
C*
C*     CHANGE HISTORY :
C*          20-NOV-87        INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_STACK.CMN'
      INCLUDE 'FIFE_UNITS.CMN'
      INCLUDE 'FIFE_SYSTEM.CMN'
      CHARACTER *80 STRING
      CHARACTER *10 BLANK
      LOGICAL CONT
C
      IERR  = 0
      LN    = 0
      LL    = LEN(LINE)
      LINE  = ' '
C
C --- WHILE CONTINUE FLAG IS SET DO...
C
C ---- CHECK STACK (FOR MACRO LINES); IF NONE, READ NEW LINE
C
10    IF (NSTACK .EQ. 0) THEN
         IF ( NREAD .EQ. 5 ) 
     $     CALL PROMPT ( NWRITE, PROMPT_STRING(1:LPROMPT))
         READ ( NREAD, 900, END=1000, ERR=2000 )STRING, BLANK
         IF ( STRING .EQ. ' ') GO TO 10
         IF ( BLANK .NE. ' ') CALL ERROR
     $          (2,'Line truncated to 80 columns.')
      ELSE
         STRING = LSTACK(NSTACK)
         NSTACK = NSTACK - 1
      ENDIF
      IF ( NREAD .NE. 5 ) WRITE(NWRITE,910) STRING(1:LENGTH(STRING))
      DO 20 J = 1,80
C
C ------ EXCLAMATION MEANS REST OF LINE IS COMMENTARY
C
         IF ( STRING(J:J) .EQ. '!' ) GO TO 25
         LN = LN + 1
         IF (LN .GT. LL) THEN
            IERR = 2
            RETURN
         ENDIF
         LINE(LN:LN) = STRING(J:J)
20       CONTINUE
25    IF ( LN .EQ. 0 ) RETURN
C
C --- NOW REMOVE ANY EXCESSIVE TRAILING BLANKS.
C
30    IF ( LINE(LN:LN) .EQ. ' ' ) THEN
         LN = LN - 1
         IF ( LN .GT. 0 ) GO TO 30
         RETURN
      ENDIF
C
C --- CHECK FOR CONTINUATION ( ELLIPSES ).
C
      CONT = .FALSE.
      IF ( LINE(LN:LN) .EQ. '.' ) THEN
         I1 = LN - 1
         IF ( LINE(I1:I1) .EQ. '.' ) THEN
C
C --- ELLIPSES FOUND, REMOVE IT AND SET CONTINUATION FLAG
C
            CONT = .TRUE.
40          LN = LN - 1
            IF ((LINE(LN:LN) .EQ. '.') .AND. (LN .GT. 1))
     $          GO TO 40
         ENDIF
      ENDIF
C
C --- IF CONTINUING, MAKE SURE THERE IS A SPACE TO SEPERATE LINES
C
      IF ( CONT ) THEN
         LN = LN + 1
         LINE(LN:LN) = ' '
         GO TO 10
      ENDIF
C
C --- END OF DO WHILE
C
      RETURN
1000  IERR = 1
      IF (LN .GT. 0) CALL ERROR(2,'End of file encountered in the '//
     $ 'middle of a statement.')
      RETURN
2000  IERR = 2
      RETURN
900   FORMAT ( A80,A10 )
910   FORMAT ( ' ',A )
      END
C
C---END READLN
C
      SUBROUTINE GETDAT ( Z, NZ, ZMIN, ZMAX )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETDAT           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET DATA
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS 207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD,  CA      94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          TO EXTRACT THE LIST OF DATA VALUES FOR THE X OR Y ARRAY.
C*
C*     OUTPUT ARGUMENTS :
C*          Z    - THE DATA ARRAY
C*          NZ   - THE NUMBER OF ENTRIES FOUND
C*          ZMIN - THE MINIMUM VALUE FOUND
C*          ZMAX - THE MAXIMUM VALUE FOUND
C*
C*     COMMON BLOCKS :
C*          FIFE_SYSTEM
C*
C*     SUBPROGRAM REFERENCES :
C*          ERROR,  RIGHT,  CATEG
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          REQUIRES 'MERLIB'
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      1-NOV-87
C*
C*     CHANGE HISTORY :
C*           1-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      CHARACTER *10 FORM
      CHARACTER *1 TYPE
      DIMENSION Z(1)
C
      NUMD = NZ
C
C --- SKIP FIRST TWO TOKEN
C
      IF (TOKEN(2) .EQ. '=') THEN
         I = 2
      ELSE
         I = 1
      ENDIF
      IF (NTOKE .LE. I) THEN
         CALL ERROR(2,'No data entered.')
         RETURN
      ENDIF
      K = 0
10    I = I + 1
      CALL CATEG ( TOKEN(I), TYPE, FORM )
      IF ( TYPE .EQ. 'I' ) THEN                      ! INTEGER
         K = K + 1
         IF (K .GT. NUMD) THEN
            CALL ERROR(2,'Too many data values.')
            RETURN
         ENDIF
         CALL RIGHT (TOKEN(I))
         READ(TOKEN(I),910) ITEMP
         Z(K) = FLOAT(ITEMP)
      ELSE IF ((TYPE .EQ. 'F') .OR. (TYPE .EQ. 'E')) THEN   ! REAL
         K = K + 1
         IF (K .GT. NUMD) THEN
            CALL ERROR(2,'Too many data values.')
            RETURN
         ENDIF
         CALL RIGHT (TOKEN(I))
         READ(TOKEN(I),900) Z(K)
      ELSE
         CALL ERROR(2,'Illegal value in data stream --> '//TOKEN(I))
      ENDIF
      IF ( I .LT. NTOKE ) GO TO 10
      IF ((NZ .GT. 0) .AND. (K .NE. NZ))
     $  CALL ERROR(2,'Number of data points may not match for X and Y.')
      NZ   = K
      ZMIN = Z(1)
      ZMAX = Z(1)
      DO 20 I = 2,NZ
         IF (Z(I) .GT. ZMAX) ZMAX = Z(I)
         IF (Z(I) .LT. ZMIN) ZMIN = Z(I)
20       CONTINUE
      RETURN
900   FORMAT(G20.0)
910   FORMAT(10X,I10)
      END
C
C---END GETDAT
C
      SUBROUTINE GETZ( Z, NZ, ZMIN, ZMAX, I )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETZ             **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET Z DATA
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS 207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA  94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          GET THIRD DIMENSIONAL DATA.
C*
C*     INPUT ARGUMENTS :
C*          I    - LOCATION IN INPUT TOKEN WHERE DATA BEGINS
C*
C*     OUTPUT ARGUMENTS :
C*          Z    - THE DATA
C*          NZ   - NUMBER OF POINTS ENTERED
C*          ZMIN - MINIMUM VALUE IN Z
C*          ZMAX - MAXIMUM VALUE IN Z
C*
C*     COMMON BLOCKS :
C*          FIFE_SYSTEM
C*
C*     SUBPROGRAM REFERENCES :
C*          CATEG, ERROR, RIGHT
C*
C*     ERROR PROCESSING :
C*          CHECKS NUMBER AND TYPES OF VALUES
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          USES 'MERLIB' ROUTINES
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0     18-FEB-86
C*
C*     CHANGE HISTORY :
C*          18-FEB-86    INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      CHARACTER *10 FORM
      CHARACTER *1 TYPE
      DIMENSION Z(1)
C
      NUMZ = NZ
      IF (NTOKE .LT. I) THEN
         CALL ERROR(2,'No data entered.')
         RETURN
      ENDIF
C
      K = 0
 10   CALL CATEG ( TOKEN(I), TYPE, FORM )
      K = K + 1
      IF (K .GT. NUMZ) THEN
         CALL ERROR(2,'Too many data values.')
         RETURN
      ENDIF
      CALL RIGHT (TOKEN(I))
      IF ( TYPE .EQ. 'I' ) THEN
         READ(TOKEN(I),910) ITEMP
         Z(K) = FLOAT(ITEMP)
      ELSE IF ((TYPE .EQ. 'F') .OR. (TYPE .EQ. 'E')) THEN
         READ(TOKEN(I),900) Z(K)
      ELSE
         CALL ERROR(2,'Illegal value in data stream --> '//TOKEN(I))
      ENDIF
      I = I + 1
      IF ( I .LE. NTOKE ) GO TO 10
C
      NZ   = K
      IF (NZ .NE. NYPTS) CALL ERROR
     $  (2,'Number of values does not match number of Y entries.')
      ZMIN = Z(1)
      ZMAX = Z(1)
      DO 20 I = 2,NZ
         IF (Z(I) .GT. ZMAX) ZMAX = Z(I)
         IF (Z(I) .LT. ZMIN) ZMIN = Z(I)
 20      CONTINUE
      RETURN
 900  FORMAT(G20.0)
 910  FORMAT(10X,I10)
      END
C
C --- END GETZ
C
      SUBROUTINE STMOUT ( STMT, IST, N )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          STMOUT           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          STATEMENT OUT
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS 207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD,  CA      94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          TO OUTPUT A STATEMENT WHICH MAY BE LONGER THAN 80 COLUMNS.
C*
C*     INPUT ARGUMENTS :
C*          STMT  - THE STATEMENT TO WRITE
C*          IST   - LENGTH OF STMT
C*          N     - LOGICAL UNIT NUMBER FOR OUTPUT
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      1-NOV-87
C*
C*     CHANGE HISTORY :
C*           1-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      CHARACTER *(*) STMT
C
      IS = 1
      IE = 77
10    IF (IE .GE. IST) THEN
         WRITE(N,900) STMT(IS:IST)
      ELSE
15       IF ( STMT(IE:IE) .NE. ',') THEN
            IE = IE - 1
            IF (IE .GT. 10) GO TO 15
         ENDIF
         WRITE(N,900) STMT(IS:IE)//'...'
         IS = IE+1
         IE = IS+76
         GO TO 10
      ENDIF
      RETURN
900   FORMAT(' ',A)
      END
C
C---END STMOUT
C
      SUBROUTINE ERROR ( ISEVER, MESS )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          ERROR            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          ERROR
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS 207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD,  CA      94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          TO PRODUCE AN ERROR MESSAGE OF WARNING, ERROR, OR FATAL
C*          SEVERITY.
C*
C*     INPUT ARGUMENTS :
C*          ISEVER - SEVERITY: 1 FOR WARNING
C*                             2 FOR ERROR
C*                             3 FOR FATAL ERROR
C*          MESS   - THE TEXT OF THE MESSAGE
C*
C*     COMMON BLOCKS :
C*          UNITS
C*
C*     FILE REFERENCES :
C*          NWRITE, NERROR
C*
C*     SUBPROGRAM REFERENCES :
C*          ENDUP,  MBELL
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      20-NOV-87
C*
C*     CHANGE HISTORY :
C*           20-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_UNITS.CMN'
      CHARACTER *(*) MESS
C
C --- BELL IS RUNG ON NWRITE IN CASE ERRORS ARE REDIRECTED.
C
      CALL MBELL ( NWRITE )
      IF (ISEVER .EQ. 1) THEN
         WRITE (NERROR, 900) MESS
      ELSE IF (ISEVER .EQ. 2) THEN
         WRITE (NERROR, 910) MESS
      ELSE
         WRITE (NERROR, 920) MESS
         CALL ENDUP
      ENDIF
      RETURN
900   FORMAT(/' * Warning, ',A/)
910   FORMAT(/' ** Error, ',A/)
920   FORMAT(/' *** Fatal Error, ',A/)
      END
C
C---END ERROR
C
      BLOCK DATA
      INCLUDE 'FIFE_SYSTEM.CMN'
      INCLUDE 'FIFE_UNITS.CMN'
      INCLUDE 'FIFE_COMMANDS.CMN'
      INCLUDE 'FIFE_MACRO.CMN'
C
      DATA NREAD/5/, NWRITE/6/, NERROR/6/
      DATA INIT_FILE/'SYS$LOGIN:FIFE.DAT'/
      DATA HELP_FILE/'SYS$HELP:FIFE.HLB'/
      DATA VERSION /'FIFE Version I.0            November 23, 1987'/
      DATA PROMPT_STRING/'FIFE> '/
      DATA LPROMPT/6/
      DATA N_EOF/6/
      END
C
C---END BLOCK DATA
C
      SUBROUTINE DOIT ( ROUTINE )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           DOIT            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DO IT
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA.   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          THIS SUBPROGRAM CONTROLS THE BRANCHING TO THE CODE WHICH
C*          IMPLEMENTS A COMMAND AFTER IT HAS BEEN VALIDATED.
C*
C*     INPUT ARGUMENTS :
C*          ICOM - THE NUMBER OF THE COMMAND TO BE EXECUTED (ALPHABETICALLY)
C*
C*     INTERNAL WORK AREAS :
C*          NONE
C*
C*     COMMON BLOCKS :
C*          NONE
C*
C*     FILE REFERENCES :
C*          NONE
C*
C*     SUBPROGRAM REFERENCES :
C*          NONE
C*
C*     ERROR PROCESSING :
C*          NONE
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      20-NOV-87
C*
C*     CHANGE HISTORY :
C*          20-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      EXTERNAL ROUTINE
C
      CALL ROUTINE
      RETURN
      END
C
C---END DOIT
C
      SUBROUTINE DISPLAY
C*
C***********************************************************************
C* 
C* NOTE!!! THE DISPLAY COMMAND IS THE ROUTINE WHICH CALLS THE NAMEIO
C* PACKAGE FOR OUTPUT VIA THE ROUTINE "WRITEC".  
C*
C***********************************************************************
C* 
C     CALL WRITEC
      RETURN
      END
C
C---END DISPLAY
C
      SUBROUTINE EDIT 
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          EDIT             **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          EDIT
C*
C*     AUTHOR :
C*          ART RAGOSTA
C*          MS207-5
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA   94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          INVOKE THE SYSTEM EDITOR
C*
C*     INPUT PARAMETER :
C*          FNAME - NAME OF FILE TO EDIT
C*
C*     COMMON BLOCKS :
C*          NONE
C*
C*     FILE REFERENCES :
C*          NONE
C*
C*     SUBPROGRAM REFERENCES :
C*          LIB$SPAWN,  ERROR
C*
C*     ERROR PROCESSING :
C*          CHECK STATUS OF SPAWN COMMAND
C*
C*     TRANSPORTABILITY LIMITATIONS :
C*          HIGHLY NON-TRANSPORTABLE
C*
C*     LANGUAGE AND COMPILER :
C*          VAX FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0      20-NOV-87
C*
C*     CHANGE HISTORY :
C*           20-NOV-87    INITIAL VERSION
C*
C***********************************************************************
C*
      CHARACTER *80 ELINE
      CHARACTER *80 FNAME
C
      CALL GETFIL ( 2, FNAME, 'SCHED' )
      ELINE = 'ADAM '//FNAME
      ISTAT = LIB$SPAWN (ELINE,,,,,,,,,,,)
      RETURN
      END
C
C---END EDIT
C
      SUBROUTINE ENDIT
      INCLUDE 'FIFE_SYSTEM.CMN'
C
      DONE = .TRUE.
      RETURN
      END
C
C---END ENDIT
C
      SUBROUTINE HARDBALL
C*
C***********************************************************************
C* 
C* NOTE!!! THE HARDBALL COMMAND IS THE ROUTINE WHICH CALLS THE NAMEIO
C* PACKAGE FOR INPUT VIA THE ROUTINE "READC".  
C*
C* "READC" (AS GENERATED BY NAMEIO) MUST BE MODIFIED TO TAKE INTO
C* ACCOUNT MACRO EXPANSION (THE STACK) AND PROMPTING.
C*
C***********************************************************************
C* 
C     CALL READC
      RETURN
      END
C
C---END HARDBALL
C
      SUBROUTINE HELPER
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          HELPER           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          HELPER
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          RETRIEVE HELP FROM A LIBRARY
C*
C*     INPUT ARGUMENTS :
C*          NONE
C*
C*     OUTPUT ARGUMENTS :
C*          NONE
C*
C*     COMMON BLOCKS :
C*          SYSTEM
C*
C*     SUBPROGRAM REFERENCES :
C*          GETEXT, HELP
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          NONE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  19-JAN-1988
C*
C*     CHANGE HISTORY :
C*          19-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_SYSTEM.CMN'
      INCLUDE 'FIFE_UNITS.CMN'
      CHARACTER *40 SUBJECT
      LOGICAL ERROR
C
      IF (NTOKE .GT. 1) THEN
         CALL GETEXT ( SUBJECT )
      ELSE
         SUBJECT = ' '
      ENDIF
      CALL HELP (HELP_FILE, SUBJECT, ERROR )
      RETURN
      END
C
C---END HELPER
C
      SUBROUTINE INCLUDER
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **         INCLUDER          **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          INCLUDER
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          BEGIN READING INPUT FROM A FILE
C*
C*     INPUT ARGUMENTS :
C*          NONE
C*
C*     OUTPUT ARGUMENTS :
C*          NONE
C*
C*     COMMON BLOCKS :
C*          UNITS
C*
C*     SUBPROGRAM REFERENCES :
C*          ERROR, GETFIL
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  19-JAN-1988
C*
C*     CHANGE HISTORY :
C*          19-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_UNITS.CMN'
      CHARACTER *80 FNAME
C
      NREAD = NREAD + 2
      CALL GETFIL ( 2, FNAME, 'DAT' )
      OPEN (UNIT=NREAD, FILE=FNAME, STATUS='OLD', ERR=100)
      RETURN
100   CALL ERROR(2,'Unable to open file '//FNAME)
      NREAD = NREAD - 2
      RETURN
      END
C
C---END INCLUDER
C
      SUBROUTINE MACRO
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           MACRO           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          MACRO
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          DEFINE A MACRO
C*
C*     INPUT ARGUMENTS :
C*          NONE
C*
C*     OUTPUT ARGUMENTS :
C*          NONE
C*
C*     COMMON BLOCKS :
C*          MACRO, SYSTEM, UNITS
C*
C*     SUBPROGRAM REFERENCES :
C*          CAPS, LEFT, PROMPT, ERROR
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  19-JAN-1988
C*
C*     CHANGE HISTORY :
C*          19-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'FIFE_MACRO.CMN'
      INCLUDE 'FIFE_SYSTEM.CMN'
      INCLUDE 'FIFE_UNITS.CMN'
      CHARACTER *132 TLINE
      CHARACTER *5 CHECK
C  
C --- MACRO
C
C ... NOTE: LEAVE THE SEVERITY LEVEL IN THE ERRORS, BELOW, AT 3.
C     SINCE WE MAY BE READING THE MACROS FROM A FILE, THERE IS NO
C     CONSISTANT WAY TO RECOVER.
C  
      IF (NTOKE .LT. 2) THEN
         CALL ERROR(2,'Macros must be named.')
         RETURN
      ENDIF    
      IF (NTOKE .GT. 2) CALL ERROR(1,'Extra entries on line ignored.')
      NMACRO = NMACRO + 1
      IF (NMACRO .GT. MAXM) CALL ERROR(3,'Too many macros.')
      MNAME(NMACRO) = TOKEN(2)
      MPT = MPTR(NMACRO)
C
C ... GET LINES OF MACRO
C
100   IF (NREAD .EQ. 5) CALL PROMPT(NWRITE,'M>')
      READ(NREAD,900,END=200) TLINE
      CALL LEFT(TLINE)
      CHECK = TLINE(1:5)
      CALL CAPS(CHECK)
      IF (CHECK .NE. 'MEND') THEN      
         MTEXT(MPT) = TLINE
         MPT = MPT + 1
         IF (MPT .GE. MAXMT) CALL ERROR(3,'Too many macro lines.')
         GO TO 100
      ENDIF
C
C ... SET POINTER TO END OF MACRO
C
200   MPTR(NMACRO+1) = MPT
      WRITE(NWRITE,910) MNAME(NMACRO)
      RETURN
900   FORMAT(A)
910   FORMAT(' ',A,' defined.')
      END
C
C---END MACRO
C
      SUBROUTINE SPAWN
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           SPAWN           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          SPAWN
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-1
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          SPAWN A SUBPROCESS TO ISSUE A DCL COMMAND
C*
C*     INPUT ARGUMENTS :
C*          NONE
C*
C*     OUTPUT ARGUMENTS :
C*          NONE
C*
C*     COMMON BLOCKS :
C*          NONE
C*
C*     SUBPROGRAM REFERENCES :
C*          GETEXT, LIB$SPAWN
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  19-JAN-1988
C*
C*     CHANGE HISTORY :
C*          19-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      CHARACTER *80 XLINE
C
      CALL GETEXT ( XLINE )
      IF (XLINE .EQ. ' ') RETURN
      ISTAT = LIB$SPAWN (XLINE,,,,,,,,,,,)
      RETURN
      END
C
C---END SPAWN
C
