      PROGRAM PAGER
c*
c*  Program to replace "#" with spaces or page breaks
c*
      INCLUDE 'PAGER.CMN'
      CHARACTER *132 LINE
C
      CALL INITIAL
C
10    READ(NREAD,900,END=100) LINE
      IF (LINE(1:1) .EQ. CHAR(12)) THEN
         IF (NLINES .EQ. 0) GO TO 10
         CALL DUMP
         IF (NLINES .GT. 0) CALL DUMP
      ELSE
         IF ((LINE(1:1) .EQ. '#') .AND. (NLINES .EQ. 0)) GO TO 10
         NLINES = NLINES + 1
         LINES(NLINES) = LINE
         IF (NLINES .GE. MAX) CALL DUMP
      ENDIF
      GO TO 10
C
100   IF (NLINES .GT. 0) CALL DUMP
      CALL EXIT
900   FORMAT(A)
      END
C
C---END PAGER
C
      SUBROUTINE INITIAL
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          INITIAL          **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          INITIAL
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-3
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          SET NUMBER OF LINES PER PAGE, STYLE, ETC.
C*
C*     COMMON BLOCKS :
C*          'PAGER.CMN'
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          14-APR-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'PAGER.CMN'
      CHARACTER *80 P(6), Q(6)
      CHARACTER *40 TEMP
      LOGICAL ERR
C
C --- NLINES = NUMBER OF LINES IN "LINES"
C --- HEAD   = TEXT OF HEADER
C --- NHEAD  = NUMBER OF LINES IN HEAD (0-4)
C --- TAIL   = TEXT OF TRAILER
C --- NTAIL  = NUMBER OF LINES IN TAIL (0-4)
C --- NUMBER = .TRUE. FOR PAGE NUMBERS
C --- DOTS   = .TRUE. TO USE DOTTED LINE INSTEAD OF SPACES
C --- MAX    = MAXIMUM NUMBER OF LINES OF TEXT ON A PAGE
C --- LPAGE  = ACTUAL LENGTH OF A PAGE
C --- NPAGE  = CURRENT PAGE NUMBER
C --- ILINES = NUMBER OF LINES PRINTED ON PAGE SO FAR
C
C --- DEFAULT VALUES:
C
      LPAGE   = 55
      NLINES  = 0
      HEAD(1) = ' '
      NHEAD   = 0
      TAIL(1) = ' '
      NTAIL   = 0
      NPAGE   = 1
      NUMBER  = .FALSE.
      DOTS    = .FALSE.
      NREAD   = 7
      NWRITE  = 8
C
C --- OVERRIDE DEFAULT VALUES:
C
      CALL GETFOR (NQ, Q, NP, P)
C
C --- INPUT FILE NAME
C
      IF (NP .LT. 1) THEN
         WRITE(6,900)
         READ(5,910) P(1)
         IF (P(1) .EQ. ' ') CALL EXIT
      ENDIF
      OPEN (UNIT=NREAD, FILE=P(1), STATUS='OLD', ERR=100)
C
C --- OUTPUT FILE SPEC
C
      IF (NP .GT. 1) THEN
         CALL PARSE (P(2),'.PAGE','FULL',P(1))
      ELSE
         CALL PARSE ('.PAGE',P(1),'FULL',P(1))
      ENDIF
      OPEN (UNIT=NWRITE, FILE=P(1), STATUS='NEW', ERR=100,
     $  CARRIAGECONTROL='LIST')
C
C --- QUALIFIERS, /PAGESIZE /NUMBER /HEAD /TAIL /DOTTED
C
      DO 10 I = 1, NQ
C
C ----- ADD PAGE NUMBERS?
C
         IF (Q(I)(1:2) .EQ. 'NU') THEN
            NUMBER = .TRUE.
C
C ----- CHANGE DEFAULT PAGE SIZE?
C
         ELSE IF (Q(I)(1:2) .EQ. 'PA') THEN
            II = INDEX(Q(I),'=')
            IF (II .EQ. 0) THEN
               WRITE(6,*)'/PAGE=nn'
               CALL EXIT
            ENDIF
            TEMP = Q(I)(II+1:)
            CALL RIGHT (TEMP)
            READ(TEMP,920) LPAGE
C
C ----- ADD HEADER LINE?
C
         ELSE IF (Q(I)(1:1) .EQ. 'H') THEN
            II = INDEX(Q(I),'=')
            IF (II .EQ. 0) THEN
               WRITE(6,*) '/HEAD="text"'
               CALL EXIT
            ENDIF
            HEAD(1) = Q(I)(II+1:)
            NHEAD = 1
C
C ---- ADD TRAILER LINE ?
C
         ELSE IF (Q(I)(1:1) .EQ. 'T') THEN
            II = INDEX(Q(I),'=')
            IF (II .EQ. 0) THEN
               WRITE(6,*) '/TAIL="text"'
               CALL EXIT
            ENDIF
            TAIL(1) = Q(I)(II+1:)
            NTAIL = 1
C
C ----- DOTTED LINE INSTEAD OF SPACES ?
C
         ELSE IF (Q(I)(1:1) .EQ. 'D') THEN
            DOTS = .TRUE.
C
         ELSE
            WRITE(6,*)'Illegal qualifier.'
            CALL EXIT
         ENDIF
10    CONTINUE
C
C --- NUMBER OF NONBLANK LINES ON PAGE
C
      MAX     = LPAGE - 2*NHEAD - NTAIL
      IF (NUMBER) MAX = MAX - 1
      IF (NUMBER .OR. (NTAIL .NE. 0)) MAX = MAX - 1
      RETURN
C
100   WRITE(6,*)'Unable to open file ',P(1)
      CALL EXIT
900   FORMAT(' Filename ? ',$)
910   FORMAT(A)
920   FORMAT(I4)
      END
C
C---END INITIAL
C
      SUBROUTINE DUMP
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           DUMP            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DUMP
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-3
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5578
C*
C*     PURPOSE :
C*          THE PAGE IS AT LEAST FULL.  EXPAND THE '#'S TO THEIR MAXIMUM
C*          OR SPECIFIED SIZE.  IF THIS TAKES LESS LINES THAN ARE 
C*          STORED, MOVE THEM UP TO START THE NEXT PAGE.
C*
C*     COMMON BLOCKS :
C*          'PAGER.CMN'
C*
C*     SUBPROGRAM REFERENCES :
C*          TOP,  BOTTOM
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     VERSION AND DATE :
C*          VERSION I.0  -  12-MAY-1988
C*
C*     CHANGE HISTORY :
C*          12-MAY-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'PAGER.CMN'
      LOGICAL LAST
C
C  TOP OF PAGE, INTIALIZE PAGE
C  COUNT NUMBER OF LINES WITH #
C  CALCULATE NUMBER OF SPACES PER POUND
C  PRINT TEXT
C  DO BOTTOM
C  REPLACE UNUSED LINES BACK IN ARRAY
C
      CALL TOP
C
C --- COUNT NUMBER OF POUND SIGNS (BREAKS)
C
      NUM_LB = 0
      DO 10 I = 1, NLINES
         IF (LINES(I)(1:1) .EQ. '#') NUM_LB = NUM_LB + 1
10    CONTINUE
C
C --- MAKE COPIES SINCE THE NUMBER CAN BE REDUCED DUE TO JUSTIFICATION
C
      ISTART = 1
      NUM_L = NUM_LB
      NLINE = NLINES
C
C --- IF THE LAST LINE IS A '#', IT CAN'T CAUSE A BREAK
C
      IF (LINES(NLINES)(1:1) .EQ. '#') THEN
         NUM_L = NUM_L - 1
         NLINE = NLINE - 1
      ENDIF
C
C --- IF NO BREAKS, JUST COPY TEXT TO OUTPUT
C
15    IF (NUM_L .EQ. 0) THEN
         DO 20 I = ISTART, NLINE
            LL = MAX0(LENGTH(LINES(I)),1)
            WRITE(NWRITE,900) LINES(I)(1:LL)
            ILINES = ILINES + 1
20       CONTINUE
      ELSE
C
C --- THERE ARE BREAKS, FIND OUT HOW LONG THEY SHOULD BE
C
C --- START WITH BREAKS 12 LINES LONG, REDUCE UNTIL ONLY 5 LINES LONG
C ---  IF STILL TOO LONG, BACKUP TO PREVIOUS BREAK AND TRY AGAIN
C
         IF (DOTS) THEN
            MULT = 5
         ELSE
            MULT = 12
         ENDIF
30       NUM = NLINE + (MULT-1)*NUM_L
         IF ((NUM .GT. MAX) .AND. (MULT .GT. 4)) THEN
            MULT = MULT - 1
            GO TO 30
         ENDIF
C
         IF (NUM .GT. MAX) THEN
C
C ----- REDUCE NUMBER OF SECTIONS ON THIS PAGE AND TRY AGAIN
C ----- NOTE: THERE MUST BE AT LEAST ONE '#' OR NUM WOULD NOT
C -----  BE GREATER THAN MAX
C
            NLINE = NLINE - 1
35          IF (LINES(NLINE)(1:1) .NE. '#') THEN
               NLINE = NLINE - 1
               GO TO 35
            ENDIF
            NUM_L = NUM_L - 1
            NLINE = NLINE - 1
            GO TO 15
         ENDIF
C
C --- OK, IT FITS ON ONE PAGE... PRINT IT OUT WITH SPACES
C
         DO 40 I = ISTART,  NLINE
            IF (LINES(I)(1:1) .EQ. '#') THEN
               IF (DOTS) THEN
                  WRITE(NWRITE,920)
                  ILINES = ILINES + 5
               ELSE
                  DO 45 J = 1, MULT
                     WRITE(NWRITE,910)
                     ILINES = ILINES + 1
45                CONTINUE
               ENDIF
            ELSE
               LL = MAX0(LENGTH(LINES(I)),1)
               WRITE(NWRITE,900) LINES(I)(1:LL)
               ILINES = ILINES + 1
            ENDIF
40       CONTINUE
      ENDIF
C
C --- IF WE PRINTED FEWER THAN ALL OF THE LINES, MOVE UNPRINTED LINES 
C ---  TO THE TOP AND SET NLINES TO THE NUMBER REMAINING
C
      NLINE = NLINE + 1
      J = 1
      IF ((NLINE .LE. NLINES) .AND. 
     $    (LINES(NLINE)(1:1) .EQ. '#')) NLINE = NLINE + 1
      DO 50 I = NLINE, NLINES
         LINES(J) = LINES(I)
         J = J + 1
50    CONTINUE
      NLINES = NLINES - NLINE + 1
      CALL BOTTOM
      RETURN
900   FORMAT(A)
910   FORMAT()
920   FORMAT(//,80('-'),//)
      END
C
C---END DUMP
C
      SUBROUTINE TOP
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **            TOP            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          TOP
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-3
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PRINT TOP OF PAGE STUFF
C*
C*     COMMON BLOCKS :
C*          'PAGER.CMN'
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          14-APR-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'PAGER.CMN'
      BYTE FF /12/
      LOGICAL INIT
      DATA INIT /.FALSE. /
C
      IF (INIT) THEN
         WRITE(NWRITE,900) FF
         ILINES = 0
      ELSE
         INIT = .TRUE.
      ENDIF
C
      IF (NHEAD .GT. 0) THEN
         WRITE(NWRITE,900) HEAD(I)(1:LENGTH(HEAD(I)))
         WRITE(NWRITE,920)
         ILINES = ILINES + 2
      ENDIF
      RETURN
900   FORMAT(A)
920   FORMAT()
      END
C
C---END TOP
C
      SUBROUTINE BOTTOM 
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          BOTTOM           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          BOTTOM
C*
C*     AUTHOR :
C*          Arthur E. Ragosta
C*          MS 219-3
C*          NASA Ames Research Center
C*          Moffett Field, Ca.  94035
C*          (415) 604-5558
C*
C*     PURPOSE :
C*          PRINT BOTTOM OF PAGE STUFF
C*
C*     COMMON BLOCKS :
C*          'PAGER.CMN'
C*
C*     SUBPROGRAM REFERENCES :
C*          LEFT,  CENTER
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          14-APR-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*
      INCLUDE 'PAGER.CMN'
      CHARACTER *78 TEMP
      CHARACTER *5 WORK
C
      IF ((NTAIL .EQ. 0) .AND. (.NOT. NUMBER)) RETURN
C
      N = LPAGE - ILINES - NTAIL       ! NO. OF BLANK LINES STILL NEEDED  
      IF (NUMBER .OR. (NTAIL .GT. 0)) N = N - 1
      DO 100 I = 1, N
         WRITE(NWRITE,900)
100      CONTINUE
C
      DO 200 I = 1, NTAIL
         WRITE(NWRITE,910) TAIL(I)(1:LENGTH(TAIL(I)))
200      CONTINUE
C
      IF (NUMBER) THEN
         WRITE(WORK,920) NPAGE
         CALL LEFT(WORK)
         TEMP = 'Page ' // WORK
         CALL CENTER (TEMP)
         WRITE(NWRITE,910) TEMP(1:LENGTH(TEMP))
      ENDIF
C
      NPAGE = NPAGE + 1
      RETURN
900   FORMAT()
910   FORMAT(A)
920   FORMAT(I5)
      END
C
C---END BOTTOM
C
