      PROGRAM POSTBUG
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          POSTBUG          **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          BUGOUT POSTPROCESSOR                    
C*
C*     AUTHOR :
C*          LAURA JURGELEIT                         
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA 94035                 
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          THIS PROGRAM WAS WRITTEN TO MERGE THE PROGRAMS VARIABLE,    
C*            COMMON, AND VERIFY.                                          
C*          DEPENDING ON OPTIONS CHOSEN, THIS PROGRAM 
C*          1.  CHECKS FORTRAN COMPILER LISTING FOR COMMON BLOCK SIZES; 
C*              COMPARING SIZES BETWEEN SUBPROGRAMS;                    
C*          2.  SEARCHES FORTRAN CROSS-REFERENCE LISTING FOR POTENTIAL  
C*              PROBLEMS SUCH AS UNINITIALIZED VARIABLES.  A GLOBAL     
C*              CROSS-REFERENCE MAP IS OPTIONALLY PRODUCED; AND         
C*          3.  VERIFIES THAT EACH COMMON BLOCK REFERENCED IN A FORTRAN 
C*              CROSS-REFERENCE LISTING IS INDEED USED IN THAT SUBPRO-  
C*              GRAM.    
C*          4.  CHECKS THAT ALL VARIABLE NAMES ARE UNIQUE WITHIN THE 
C*              FIRST SIX CHARACTERS         
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FLAGS
C*
C*     SUBPROGRAM REFERENCES :
C*          INIT, FNDCRD, FNDCODE, PSECT, GETSUB, PROSUB, MERGE
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     COMMON VERSION :
C*          VERSION I.5        
C*
C*     COMMON CHANGE HISTORY :
C*          08-JUL-88     CHANGED MAXIMUM NUMBER OF CHARACTERS FOR 
C*                         NAMES TO 12
C*          12-NOV-85     FIXED TO PROPERLY HANDLE BLOCK DATA
C*          30-JUL-85     FILENAME ADDED TO PRINTOUT
C*          06-MAR-85     SAVE/RESTORE OPTION ADDED
C*          05-JUN-84     SORT OPTION ADDED
C*          30-JAN-84     INITIAL VERSION
C*
C*     VARIABLE VERSION :
C*          VERSION I.1     
C*
C*     VARIABLE CHANGE HISTORY :
C*          07-NOV-85    DATA DICTIONARY ADDED
C*          30-JUL-85    INITIAL VERSION
C*
C*     VERIFY VERSION :
C*          VERSION I.0             
C*
C*     VERIFY CHANGE HISTORY :
C*          AUGUST 26, 1986         INITIAL VERSION 
C*
C*     CHANGE HISTORY :
C*          04-SEP-91    NAMELIST OPTION ADDED
C*          29-NOV-90    FIXED BUG IN COMMON CHECKING;
C*                        CHANGED COMMON CHECK OUTPUT DISPLAY
C*          22-AUG-90    FIXED BUG IN VERIFY CHECKING
C*          15-JUN-90    ADDED VAX SPEC. STMT. PROCESSING
C*          16-DEC-88    INCREASED MAX NUMBER OF REFERENCES TO
C*                         VARIABLE/ARRAY FROM 200 TO 225
C*          09-DEC-88    INCREASED NUMBER OF COMMON BLOCKS FROM 100 TO 150
C*          23-NOV-87    MODIFIED TO PRODUCE SUMMARY WHEN NO OPTIONS CHOSEN
C*          09-OCT-86    INITIAL VERSION
C*
C***********************************************************************
C*
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      CHARACTER *150 OPSTRG
      CHARACTER *15 SAVNAM
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT,
     $        SAVE, RESTOR, BD, Q1Q, NML, COM, DONE, GO, DID, UCHECK,
     $        NMLST
C
      CALL INIT
C
C --- NO POSTBUG OPTIONS CHOSEN
C
      IF ((.NOT. RUNVFY) .AND. (.NOT. RUNVBL) .AND. (.NOT. RUNC)) THEN
         CALL SUMMAR
         STOP
      ENDIF
C
C --- IF VERIFY OR VARIABLE, FIND 1ST EXEC. CODE
C
10    IF ( RUNVFY .OR. RUNVBL ) THEN
         CALL FNDCRD
C
C --- IF BLOCK DATA, NO NEED TO CHECK FOR EXECUTABLE CODE
C
         IF ( BD ) GO TO 15
         CALL FNDCODE
      ENDIF
C
C --- PROCESS 'PROGRAM SECTIONS'
C
15    CALL PSECT 
      CALL GETSUB
      IF (RUNVBL .OR. RUNVFY) THEN
         CALL PROSUB         
         IF ( RUNVBL .AND. (.NOT. BD)) CALL MERGE
      ENDIF
      CALL MESSG
      GO TO 10
      END
C
C---END POSTBUG
C
      SUBROUTINE INIT  
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          INIT             **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          INITIALIZATION                          
C*
C*     AUTHOR :
C*          LAURA JURGELEIT                         
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA  94035                
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          INITIALIZE PROGRAM-WIDE DATA                                
C*
C*     COMMON BLOCKS :
C*          FLIST, FILES, VCOM, COMMNS, SUBPRS, RUNOPT
C*
C*     FILE REFERENCES :
C*          NSRCD - FILENAME OF SOURCE CODE, SET TO 90
C*          NOPT  - OPTIONS FILE, SET TO 91
C*          NOUT  - SYSTEM PRINTOUT, SET TO 93
C*          NIN   - FORTRAN LISTING FILE, SET TO 94
C*          NDICT - VARIABLE DICTIONARY, SET TO 98
C*          NSUM  - SUMMARY PRINTOUT, SET TO 96
C*          NRSTR - SAVE/RESTORE FILE, SET TO 95
C*          NVFIL - ERROR MESSAGES FOR VARIABLE CHECKING, SET TO 97
C*
C*     SUBPROGRAM REFERENCES :
C*          OPTION, GETLIN
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          24-SEP-86    INITIAL VERSION
C*
C***********************************************************************
C*                         
      PARAMETER (MAXSUB=400,MAXVAR=3000,MAXCOM=150,MXCHR=12)
      COMMON / FLIST / LINE, EOF, FF
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / VCOM / NUMVAR, VARNAM(MAXVAR), VARTYP(MAXVAR),
     $                VARSIZ(MAXVAR), VCOMMN(MAXVAR), XREF(MAXVAR)
      COMMON / COMMNS / NAMES(MAXCOM), LENS(MAXCOM), ILENS(MAXCOM), 
     $                  NCERR, NUMCOM, NCOMMN
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      CHARACTER *(MXCHR) VARNAM, SUBNAM, VCOMMN, SNAME, NAMES 
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE 
      CHARACTER *100 CHART
      CHARACTER *15 SAVNAM
      CHARACTER *1 VARTYP, FF
      CHARACTER *(MAXSUB) XREF
      INTEGER VARSIZ
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF
      DATA FF/12/
C      DATA SNAME /'$$$$$$$$$$$$'/
      DO 5 I = 1,MXCHR
         SNAME(I:I) = '$'
5     CONTINUE
C
C --- UNIT NUMBERS
C
      NSRCD = 90
      NOPT  = 91
      NOUT  = 93
      NIN   = 94
      NSUM  = 96
      NRSTR = 95
      NVFIL = 97
      NDICT = 98
C
C --- COUNTERS, FLAGS
C
      EOF   = .FALSE.
      Q1Q   = .FALSE.
      NUMVAR = 0
      NUMCOM = 0
      NUMSUB = 1
C
      CALL OPTION
      IF ( MAPC ) THEN
         DO 10 I = 1, MAXSUB
            CHART(I) = ' '
10       CONTINUE
      ENDIF
      IF ( MAPV ) THEN
         DO 20 J = 1, MAXVAR
            XREF(J) = ' '
20       CONTINUE
      ENDIF     
C
C --- GET FIRST LINE
C
      CALL GETLIN
      RETURN
      END
C
C---END INIT  
C
      SUBROUTINE OPTION
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          OPTION           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET OPTION                              
C*
C*     AUTHOR :
C*          LAURA JURGELEIT                         
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA  94035                
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          READS THE OPTIONS FROM THE OPTIONS FILE.                    
C*
C*     COMMON BLOCKS :
C*          FILES, RUNOPT
C*
C*     FILE REFERENCES :
C*          NOPT, NIN
C*
C*     SUBPROGRAM REFERENCES :
C*          CAPS, RESTR
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          01-19-87     VARIABLE UNIQUENESS CHECKING ADDED
C*          11-18-87     OPEN STATEMENTS W/ 'LIST' CARRIAGECONTROL 
C*                        ADDED
C*          08-OCT-86    MODIFIED FOR POSTBUG
C*          23-SEP-86    INITIAL VERSION
C*
C***********************************************************************
C*
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLIST / LINE, EOF, FF
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *15 SAVNAM
      CHARACTER *1 FF
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, EOF, UCHECK, NMLST
C
C --- DEFAULT OPTIONS
C
      RUNC   = .TRUE.
      MAPC   = .FALSE.
      SORT   = .TRUE.
      SAVE   = .FALSE.
      RESTOR = .FALSE.
      RUNVFY = .FALSE.
      RUNVBL = .TRUE.
      UCHECK = .FALSE.
      LFIRST = .TRUE.
      MAPV   = .FALSE.
      DICT   = .FALSE.
      NMLST  = .FALSE.
      CALL LIB$SET_SYMBOL ('BUG_DV', '0')
      NPRINT = 1
      SAVNAM = 'COMSAVE.DAT'
C
      OPEN (UNIT=NOUT,STATUS='NEW',CARRIAGECONTROL='LIST',ERR=1000)
      OPEN (UNIT=NVFIL,STATUS='NEW',CARRIAGECONTROL='LIST',ERR=1010)
      WRITE (NVFIL,905) FF
      OPEN (UNIT=NIN,STATUS='OLD',ERR=20)
      OPEN (UNIT=NOPT,STATUS='OLD',ERR=20)
5     READ (NOPT,900,END=20) OPSTRG
      CALL CAPS (OPSTRG)
C
C --- DO COMMON BLOCK VERIFICATION?
C
      IF ( (INDEX(OPSTRG,'VERIFY') .NE. 0) .AND.
     $     (INDEX(OPSTRG,'NOVERIFY') .EQ. 0)) RUNVFY = .TRUE.
C
C --- PROCESS SUBPROGRAMS CONTAINING NAMELIST STATEMENTS ?
C
      IF ( (INDEX(OPSTRG,'NAMELIST') .NE. 0) .AND.
     $     (INDEX(OPSTRG,'NONAMELIST') .EQ. 0)) NMLST = .TRUE.
C
C --- COMMON OPTIONS
C
      IF (INDEX(OPSTRG,'NOCOMM') .NE. 0) RUNC = .FALSE.
      IF ( (INDEX(OPSTRG,'MAPCO') .NE. 0) .AND.
     $     (INDEX(OPSTRG,'NOMAPCO') .EQ. 0) ) MAPC = .TRUE.
      IF (INDEX(OPSTRG,'NOSORT') .NE. 0) SORT = .FALSE.
      IF (INDEX(OPSTRG,'SAVE') .NE. 0) SAVE = .TRUE.  
      IF (INDEX(OPSTRG,'RESTORE') .NE. 0) RESTOR = .TRUE.
      IF (INDEX(OPSTRG,'PRINT=') .NE. 0) THEN
         N      = INDEX(OPSTRG,'PRINT=') + 6
         IF (N .NE. 6) THEN
            NPRINT = ICHAR(OPSTRG(N:N)) - ICHAR('0')
            IF (NPRINT .GT. 2 .OR. NPRINT .LT. 0) NPRINT = 1
         ENDIF
      ENDIF
      IF (INDEX(OPSTRG,'SAVENAME=') .NE. 0) THEN
         N      = INDEX(OPSTRG,'SAVENAME=')
         IF (N .NE. 0) THEN
            I = N+10
            N = I
10          N = N + 1
            IF (OPSTRG(N:N) .EQ. '''') THEN
               SAVNAM = OPSTRG(I:N-1)
               GO TO 20
            ENDIF
            IF (N .LT. 132) GO TO 10
         ENDIF      
      ENDIF
C
C --- VARIABLE OPTIONS
C
      IF (INDEX(OPSTRG,'NOVAR') .NE. 0) RUNVBL = .FALSE.
      IF (INDEX(OPSTRG,'NOFIRST') .NE. 0) LFIRST = .FALSE.
      IF ( (INDEX(OPSTRG,'MAPVAR') .NE. 0) .AND.
     $     (INDEX(OPSTRG,'NOMAPV') .EQ. 0) ) MAPV = .TRUE.
      IF ( (INDEX(OPSTRG,'DICT') .NE. 0) .AND.
     $     (INDEX(OPSTRG,'NODICT') .EQ. 0) ) DICT = .TRUE.
      IF ( (INDEX(OPSTRG,'UCHECK') .NE. 0) .AND.
     $     (INDEX(OPSTRG,'NOUCHECK') .EQ. 0) ) UCHECK = .TRUE.
      GO TO 5
C
C --- REBUILD OPSTRG FOR OUTPUT
C
20    OPSTRG = ' '
      IOP    = 1
      IF ( RUNVFY ) THEN
         OPSTRG(IOP:IOP+7) = 'VERIFY, '
         IOP = IOP + 8
      ELSE
         OPSTRG(IOP:IOP+9) = 'NOVERIFY, '
         IOP = IOP + 10
      ENDIF
      IF ( NMLST ) THEN
         OPSTRG(IOP:IOP+9) = 'NAMELIST, '
         IOP = IOP + 10
      ELSE
         OPSTRG(IOP:IOP+11) = 'NONAMELIST, '
         IOP = IOP + 12
      ENDIF
      IF ( RUNC ) THEN
         OPSTRG(IOP:IOP+7) = 'COMMON, '
         IOP = IOP + 8
      ELSE
         OPSTRG(IOP:IOP+9) = 'NOCOMMON, '
         IOP = IOP + 10
      ENDIF
      IF ( MAPC ) THEN
         OPSTRG(IOP:IOP+10) = 'MAPCOMMON, '
         IOP = IOP + 11
      ELSE
         OPSTRG(IOP:IOP+12) = 'NOMAPCOMMON, '
         IOP = IOP + 13
      ENDIF
      IF ( SORT ) THEN
         OPSTRG(IOP:IOP+5) = 'SORT, '
         IOP = IOP + 6
      ELSE
         OPSTRG(IOP:IOP+7) = 'NOSORT, '
         IOP = IOP + 8
      ENDIF
      IF ( SAVE ) THEN
         OPSTRG(IOP:IOP+5) = 'SAVE, '
         IOP = IOP + 6
      ENDIF
      IF ( RESTOR ) THEN
         OPSTRG(IOP:IOP+8) = 'RESTORE, '
         IOP = IOP + 9
      ENDIF
      OPSTRG(IOP:IOP+5) = 'PRINT='
      IOP = IOP + 6
      OPSTRG(IOP:IOP) = CHAR(ICHAR('0')+NPRINT)
      IOP = IOP + 1
      OPSTRG(IOP:IOP+28) = ', SAVENAME=''' // SAVNAM // ''''
      IOP = IOP + 29
      IF ( RUNVBL ) THEN
         OPSTRG(IOP:IOP+11) = ', VARIABLE, '
         IOP = IOP + 12
         IF ( UCHECK ) THEN
            OPSTRG(IOP:IOP+7) = 'UCHECK, '
            IOP = IOP + 8
         ELSE
            OPSTRG(IOP:IOP+9) = 'NOUCHECK, '
            IOP = IOP + 10
         ENDIF
      ELSE
         OPSTRG(IOP:IOP+13) = ', NOVARIABLE, '
         IOP = IOP + 14
      ENDIF
      IF ( LFIRST ) THEN
         OPSTRG(IOP:IOP+6) = 'FIRST, '
         IOP = IOP + 7
      ELSE
         OPSTRG(IOP:IOP+8) = 'NOFIRST, '
         IOP = IOP + 9
      ENDIF
      IF ( MAPV ) THEN
         OPSTRG(IOP:IOP+13) = 'MAPVARIABLES, '
         IOP = IOP + 14
      ELSE
         OPSTRG(IOP:IOP+15) = 'NOMAPVARIABLES, '
         IOP = IOP + 16
      ENDIF
      IF ( DICT ) THEN
         OPSTRG(IOP:IOP+9) = 'DICTIONARY'
      ELSE
         OPSTRG(IOP:IOP+11) = 'NODICTIONARY'
      ENDIF
      IF ( RESTOR ) CALL RESTR
      RETURN
1000  WRITE (6,999)
      RETURN
1010  WRITE (NOUT,998)
      RETURN
900   FORMAT ( A132 )
905   FORMAT (A)
998   FORMAT (' ***** ERROR OPENING OUTPUT FILE (UNIT=12).')
999   FORMAT (' ***** ERROR OPENING OUTPUT FILE (UNIT=6).')
      END
C
C---END OPTION
C
      SUBROUTINE SAVER
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          SAVER            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          SAVE 
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF 94035              
C*          (415)965-5578                           
C*
C*     PURPOSE :
C*          SAVE STATUS FOR POSSIBLE RESTORE LATER.
C*
C*     COMMON BLOCKS :
C*          RUNOPT, COMMNS, FILES
C*
C*     FILE REFERENCES :
C*          NRSTR
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          26-DEC-84     INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXCOM=150,MXCHR=12)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / COMMNS / NAMES(MAXCOM), LENS(MAXCOM), ILENS(MAXCOM), 
     $                  NCERR, NUMCOM, NCOMMN
      CHARACTER *150 OPSTRG
      CHARACTER *15 SAVNAM
      CHARACTER *(MXCHR) NAMES
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
C
      OPEN ( UNIT=NRSTR, STATUS='NEW', FILE=SAVNAM,
     $  FORM='UNFORMATTED', ERR=1000 )
      WRITE ( NRSTR ) NUMCOM
      DO 100 I = 1, NUMCOM
         WRITE ( NRSTR ) NAMES(I), LENS(I)
100      CONTINUE
      CLOSE ( UNIT=NRSTR )
      RETURN
1000  WRITE ( NOUT, 900 )
      RETURN
900   FORMAT ( ' ****** ERROR, UNABLE TO OPEN SAVE FILE.')
      END
C
C---END SAVER
C
      SUBROUTINE RESTR
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          RESTR            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          RESTORE
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF 94035              
C*          (415)965-5578                           
C*
C*     PURPOSE :
C*          RESTORE THE STATUS OF A PREVIOUSLY SAVED JOB.
C*
C*     COMMON BLOCKS :
C*          RUNOPT, COMMNS, FILES
C*
C*     FILE REFERENCES :
C*          NRSTR 
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          26-DEC-84     INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXCOM=150,MXCHR=12)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / COMMNS / NAMES(MAXCOM), LENS(MAXCOM), ILENS(MAXCOM), 
     $                  NCERR, NUMCOM, NCOMMN
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      CHARACTER *150 OPSTRG
      CHARACTER *15 SAVNAM
      CHARACTER *(MXCHR) NAMES
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
C
      OPEN ( UNIT=NRSTR, STATUS='OLD', FILE=SAVNAM,
     $  FORM='UNFORMATTED', ERR=1000 )
      READ ( NRSTR ) NUMCOM
      DO 100 I=1,NUMCOM
         READ ( NRSTR ) NAMES(I), LENS(I)
100      CONTINUE
      CLOSE ( UNIT=NRSTR )
      RETURN
1000  WRITE ( NOUT, 900 )
      RETURN
900   FORMAT ( ' ***** ERROR, UNABLE TO OPEN RESTORE FILE.')
      END
C
C---END RESTR
C
      SUBROUTINE FNDCRD
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          FNDCRD           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          FIND SUBPROGRAM CARD
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA.   94035              
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          SKIP THROUGH THE LISTING UNTIL A NEW SUBPROGRAM IS FOUND
C*          ( LOOK FOR PROGRAM, SUBROUTINE, OR FUNCTION CARD )
C*          
C*     COMMON BLOCKS :
C*          FLIST, FLAGS, FORSTM
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN, INDEX, RIGHT, GETSTMT
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          NOVEMBER 18, 1987       COLUMN NUMBERS PARAMETERIZED
C*          AUGUST 12, 1987         MODIFIED TO GET ENTIRE FIRST
C                                   STATEMENT IN SUBPROGRAM
C*          OCTOBER 8, 1986         MODIFIED TO NOT SKIP BLOCK DATA
C*          AUGUST 26, 1986         INITIAL VERSION 
C*           
C*
C***********************************************************************
C*
      COMMON / FLIST / LINE, EOF, FF
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      COMMON / FORSTM / STMT, CARD, LINUM, NUMSTM, CONT, NSTART, NCOLST
      CHARACTER *3000 STMT
      CHARACTER *132 LINE
      CHARACTER *80 CARD
      CHARACTER *14 CNUM
      CHARACTER *1 FF
      LOGICAL EOF, BD, COM, DONE, GO, DID, NML, CONT
      DATA NCOLST /9/
C 
      BD = .FALSE.
      ICOL7 = NCOLST + 6
C
C --- SKIP COMMENTS
C                                                                
10    IF ( LINE(NCOLST:NCOLST) .EQ. 'C' 
     $     .OR. LINE(NCOLST:NCOLST) .EQ. '*' ) THEN
         CALL GETLIN
         GO TO 10
      ENDIF
      IF (( INDEX(LINE(ICOL7:ICOL7+64),'BLOCK DATA') .NE. 0) .OR. 
     $    ( INDEX(LINE(ICOL7:ICOL7+64),'BLOCKDATA') .NE. 0)) THEN
         BD = .TRUE.
         RETURN
      ENDIF
      IF ( ( INDEX(LINE(ICOL7:ICOL7+64),'PROGRAM') .EQ. 0 ) .AND.
     $     ( INDEX(LINE(ICOL7:ICOL7+64),'SUBROUTINE') .EQ. 0 ) .AND.
     $     ( INDEX(LINE(ICOL7:ICOL7+64),'FUNCTION') .EQ. 0 ) ) THEN
20         CALL GETLIN
           GO TO 10
      ENDIF
C
C --- CHECK TO SEE THAT THE COLUMNS BEFORE KEYWORD CONTAIN DIGITS ONLY
C
      CNUM = LINE(1:ICOL7-1)
      CALL RIGHT ( CNUM )
C 
C --- IF COLUMNS BEFORE KEYWORD DO NOT CONTAIN ONLY DIGITS, THIS IS NOT
C --- A SUBPROGRAM CARD...GET ANOTHER LINE
C
      READ(CNUM,900,ERR=20) NUM
C
C --- THIS IS FIRST CARD OF SUBPROGRAM, GET ENTIRE STATEMENT
C
      CARD = LINE(1:NCOLST+71)
      LINUM = NUM
      CALL GETSTMT
      RETURN
900   FORMAT(I14)
      END
C
C---END FNDCRD
C
      SUBROUTINE GETLIN
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **         GETLIN            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET LINE
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA.   94035              
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          GET A SINGLE LINE FROM THE LISTING FILE.
C*
C*     METHODOLOGY:
C*          BASED ON SUBPROGRAM GETLIN IN PROGRAM VARIABLE
C*          BY ART RAGOSTA
C*
C*     COMMON BLOCKS :
C*          FLIST, FILES, RUNOPT      
C*
C*     SUBPROGRAM REFERENCES :
C*          SUMMAR, SAVER, CHARTV, CHARTR, DICOUT, UNTAB, FATAL
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          OCTOBER 7, 1986         MODIFIED FOR POSTBUG
C*          AUGUST 26, 1986         INITIAL VERSION
C*           
C*
C***********************************************************************
C*
      PARAMETER (NHEAD=4,NPLUS=3)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLIST / LINE, EOF, FF
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *15 SAVNAM
      CHARACTER *1 FF
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT,
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF
C
C --- IF WE HAVE REACHED END OF FILE OR 'Q1Q' ROUTINES INSERTED BY 
C --- PREBUG, START END OF PROGRAM PROCESSING
C
      IF ( EOF .OR. Q1Q ) THEN
         CALL SUMMAR
         IF ( UCHECK ) CALL UVCHK
         IF ( SAVE ) CALL SAVER
         IF ( MAPV ) CALL CHARTV
         IF ( MAPC ) CALL CHARTR
         IF ( DICT ) CALL DICOUT
         CLOSE (UNIT=NOUT)
         CLOSE (UNIT=NVFIL)
         STOP
      ENDIF
      READ (NIN,900,END=2000,ERR=1000) LINE
C
C --- SKIP PAGE HEADER IF NECESSARY
C
      IF (INDEX(LINE,FF) .NE. 0) THEN
         DO 10 I = 1,NHEAD
            READ(NIN,900,END=2000,ERR=1000) LINE
10       CONTINUE
      ENDIF
      CALL UNTAB(LINE)
      CALL CAPS(LINE)
C
C --- CHECK FOR END OF (RELEVANT) FILE; E.G. '+' IN COL. 3
C
      IF (LINE(NPLUS:NPLUS) .EQ. '+') GO TO 2000
      RETURN
C
1000  CALL FATAL('Error reading listing file.',0)
2000  EOF = .TRUE.
      RETURN
900   FORMAT(A)
      END
C
C---END GETLIN
C
      SUBROUTINE FNDCODE
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **         FNDCODE           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          FIND CODE
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA.   94035              
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          FIND LINE NUMBER OF THE FIRST EXECUTABLE CODE IN THIS 
C*          SUBPROGRAM 
C*
C*     COMMON BLOCKS :
C*          FORSTM, FLAGS
C*
C*     SUBPROGRAM REFERENCES :
C*          GETSTMT
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          JUNE 15, 1990           ADDED VAX SPEC. STATEMENTS
C*          AUGUST 12, 1987         MODIFIED TO CHECK FOR NAMELIST
C*          AUGUST 26, 1986         INITIAL VERSION
C*           
C*
C***********************************************************************
C*
      PARAMETER (NSPEC=14,NVSPEC=9)
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      COMMON / FORSTM / STMT, CARD, LINUM, NUMSTM, CONT, NSTART, NCOLST
      CHARACTER *3000 STMT
      CHARACTER *80 CARD
      CHARACTER *6 SPEC(NSPEC)
      CHARACTER *8 VSPEC(NVSPEC)
      LOGICAL CONT, GO, COM, DONE, DID, NML, BD
      DATA SPEC/      
     $ 'CHARAC', 'COMMON', 'COMPLE', 'DIMENS', 'DOUBLE', 
     $ 'EQUIVA', 'EXTERN', 'IMPLIC', 'INCLUD', 'INTEGE', 
     $ 'INTRIN', 'LOGICA', 'REAL  ', 'SAVE  '/
      DATA VSPEC/
     $ 'BYTE    ', 'DEFINEFI', 'ENDMAP  ', 'ENDSTRUC', 'ENDUNION',
     $ 'MAP     ', 'RECORD  ', 'STRUCTUR', 'UNION   '/
C
      NML = .FALSE.
      GO = .TRUE.
10    CALL GETSTMT
      LENST = LENGTH (STMT)
C
      IF (STMT(1:4) .EQ. 'DATA') THEN
         ISL = INDEX(STMT,'/')
         IEQ = INDEX(STMT,'=')
         IF (IEQ .LT. ISL) GO TO 10
      ELSEIF ((STMT(1:3) .EQ. 'END').AND.(LENST .EQ. 3)) THEN
         GO = .FALSE.                                  
      ELSEIF ((STMT(1:8) .EQ. 'NAMELIST')) THEN
         NML = .TRUE.
         GO TO 10
      ELSE
         DO 20 I =  1,NSPEC
            IF (((INDEX (STMT(1:LENST),SPEC(I)(1:LENGTH(SPEC(I))))
     $                  .NE. 0) .AND. (INDEX (STMT,'=') .EQ. 0))
     $         .OR. (STMT(1:10) .EQ. 'PARAMETER(')) GO TO 10
20       CONTINUE 
         DO 30 I = 1,NVSPEC
            IF ((INDEX(STMT(1:LENST),VSPEC(I)(1:LENGTH(VSPEC(I))))
     $           .NE. 0) .AND. (INDEX (STMT,'=') .EQ. 0)) GO TO 10
30       CONTINUE
      ENDIF
      NSTART = NUMSTM
      RETURN
      END
C
C--- END FNDCODE
C
      SUBROUTINE GETSTMT
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **         GETSTMT           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET STATEMENT
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA.   94035              
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          RETRIEVE THE NEXT, FULL FORTRAN STATEMENT
C*          CAPITALIZING AND REMOVING CHARACTER STRINGS
C*          
C*     METHODOLOGY :
C*          BASED ON SUBPROGRAM GETSTM IN PROGRAM CALLS BY
C*          ART RAGOSTA
C*
C*     COMMON BLOCKS :
C*          FORSTM
C*
C*     SUBPROGRAM REFERENCES :
C*          GETCRD
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          AUGUST 26, 1986         INITIAL VERSION 
C*           
C*
C***********************************************************************
C*
      PARAMETER (MXLEN = 3000)
      COMMON / FORSTM / STMT, CARD, LINUM, NUMSTM, CONT, NSTART, NCOLST
      CHARACTER *(MXLEN) STMT, TSTMT
      CHARACTER *80 CARD
      LOGICAL CONT
C
      STMT = ' '
      TSTMT = ' '
      IPTR = 1
      LENTST = 0
      ICOL7 = NCOLST + 6
C
C --- COPY ONLY NON-BLANK CHARACTERS
C
      NUMSTM = LINUM
10    DO 20 I = ICOL7,ICOL7+64
         IF (CARD(I:I) .NE. ' ') THEN
            LENTST = LENTST + 1
            IF (LENTST .GT. MXLEN) GO TO 25
            TSTMT(LENTST:LENTST) = CARD(I:I)
         ENDIF        
20    CONTINUE
      CALL GETCRD
C
C --- CONTINUE COPYING IF THERE IS A CONTINUATION CARD
C
      IF (CONT) GO TO 10
C
C --- RE-READ STATEMENT, DELETING ALL CHARACTER STRINGS
C
25    CALL STRPSTM (TSTMT, LENTST, STMT, IPTR)
      RETURN
      END
C
C---END GETSTM
C
      SUBROUTINE GETCRD
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **         GETCRD            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET CARD
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA.   94035              
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          GET THE NEXT NON-COMMENT CARD FROM THE LISTING.
C*          
C*     METHODOLOGY :
C*          BASED ON SUBPROGRAM GETCRD IN PROGRAM CALLS BY
C*          ART RAGOSTA
C*
C*     COMMON BLOCKS :
C*          FORSTM, FLIST, FILES
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          AUGUST 26, 1986         INITIAL VERSION 
C*           
C*
C***********************************************************************
C*
      PARAMETER (NHEAD=4)
      COMMON / FORSTM / STMT, CARD, LINUM, NUMSTM, CONT, NSTART, NCOLST
      COMMON / FLIST / LINE, EOF, FF
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      CHARACTER *3000 STMT
      CHARACTER *132 LINE
      CHARACTER *80 CARD
      CHARACTER *1 FF
      LOGICAL CONT, EOF
C
      CONT = .FALSE.
10    READ (NIN,900) CARD
C
C --- SKIP "FF" IF NECESSARY
C 
      IF (INDEX(CARD,FF) .NE. 0) THEN
        DO 15 I = 1,NHEAD
           READ(NIN,900) CARD
15      CONTINUE
      ENDIF
C
C --- ERROR READING LINUM BELOW PROBABLY MEANS WE'VE RUN INTO A
C COMPILATION ERROR, E.G., %FOR-E-errormsg.  JUST SKIP THE LINE.
C
      READ (CARD(1:5),910,ERR=10) LINUM
      IF (CARD(NCOLST:NCOLST) .EQ. 'C' .OR.
     $    CARD(NCOLST:NCOLST) .EQ. '*') GO TO 10
      IF (CARD(NCOLST+5:NCOLST+5) .NE. ' ') CONT = .TRUE.
      RETURN
900   FORMAT ( A80 ) 
910   FORMAT ( I5 )
      END
C 
C---END GETCRD
C
      SUBROUTINE PSECT 
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          PSECT            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          PROGRAM SECTIONS                        
C*
C*     AUTHOR :
C*          LAURA JURGELEIT                         
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA 94035                 
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          GET THE NAMES AND NUMBERS OF THE COMMON BLOCKS FROM THE     
C*          'PROGRAM SECTIONS' PART OF THE LISTING.
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FLIST, COMMNS, VERCOM, ONESUB, ONEVAR, FLAGS
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN, COMFND
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          18-NOV-87    MODIFIED TO SEARCH FOR FIELDS IN LISTING
C*          07-OCT-86    MODIFIED FOR POSTBUG
C*          29-SEP-86    INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXVPS=1000,MAXCOM=150,MREF=225,MXCHR=12,MXCSUB=60)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLIST / LINE, EOF, FF
      COMMON / COMMNS / NAMES(MAXCOM), LENS(MAXCOM), ILENS(MAXCOM), 
     $                  NCERR, NUMCOM, NCOMMN
      COMMON / VERCOM / CNAMES(MAXCOM), NUMS(MAXCOM), NTCOM, NVFERR
      COMMON / ONESUB / SNAMES(MAXVPS), STYPE(MAXVPS), SIZE(MAXVPS),
     $                  VCOMM(MAXVPS), SCOMMN(0:MXCSUB), NSVAR
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      CHARACTER *(MXCHR) SNAMES, VCOMM, SCOMMN, CNAME, NAMES, CNAMES,
     $ BLANK, VNAME
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE 
      CHARACTER *15 SAVNAM
      CHARACTER *10 REFS
      CHARACTER *7 SLEN
      CHARACTER *6 TNUM
      CHARACTER *3 PRSECT
      CHARACTER *1 FF, STYPE, TYPE, CFLAG
      INTEGER SIZE
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT,
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF, COM, DONE, GO, DID, NML, BD
      DATA BLANK /'            '/
C
      NCOM = 0    
      COM = .FALSE.
10    CALL GETLIN
C
C --- LOOK FOR 'PROGRAM SECTIONS' HEADING
C
      IPS = INDEX(LINE,'PROGRAM SECTIONS')
      IF (IPS .EQ. 0) THEN
         GO TO 10
      ELSE IF (IPS .NE. 1) THEN
C
C --- MAKE SURE WE ARE IN THE RIGHT PLACE
C --- (I.E., MAKE SURE THE REST OF THE LINE IS ' ')
C
         IF ((LINE(1:IPS) .NE. ' ').OR.(LINE(IPS+16:132) .NE. ' '))
     $        GO TO 10
      ENDIF
C
C --- SKIP DOWN TO FIELD TITLES
C
15    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 15
      INAM = INDEX(LINE,'NAME')
      IBYT = INDEX(LINE,'BYTES')
C
C --- SKIP DOWN TO DATA
C
20    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 20
C
C --- GET NUMBERS AND NAMES OF COMMON BLOCKS
C
30    CNAME = LINE(INAM:INAM+MXCHR-1)     
C
C --- SECTION ENDS WITH BLANK LINE, OR WHEN 'Q1Q' REACHED
C
      IF (( CNAME .NE. BLANK) .AND. (CNAME(1:3) .NE. 'Q1Q')) THEN
         TNUM = LINE(1:INAM-2)
         CALL RIGHT (TNUM)
         READ (TNUM,900) NCOMMN
         IF (NCOMMN .GT. MXCSUB) CALL FATAL
     $      ('TOO MANY COMMON BLOCKS IN ONE SUBPROGRAM',MXCSUB)
         SCOMMN(NCOMMN) = CNAME
         IF ((CNAME(1:1) .NE. '$') .OR. (CNAME .EQ. '$BLANK')) THEN
            IF ( .NOT. COM ) COM = .TRUE.
            IF (NCOM .EQ. MAXCOM) CALL FATAL
     $      ('NUMBER OF COMMON BLOCKS EXCEEDS PROGRAM DIMENSIONS',
     $        MAXCOM)
            NCOM = NCOM + 1           
            NUMS(NCOM) = NCOMMN
            CNAMES(NCOM) = CNAME
            SLEN = LINE(IBYT-2:IBYT+4)
            READ(SLEN,900) ILEN
            IF ( RUNC ) CALL COMFND (CNAME, ILEN)
         ENDIF
         CALL GETLIN
         GO TO 30
      ENDIF
      NTCOM = NCOM
      RETURN
900   FORMAT(I)
      END
C                                     
C---END PSECT 
C
      SUBROUTINE COMFND ( CNAME, ILEN )
C*                        
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          COMFND           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          COMMON FOUND
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF 94035              
C*          (415)965-5578                           
C*
C*     PURPOSE :
C*          COMPARE THE NAME TO THE LIST OF NAMES, ADD IF NEW,
C*           ERROR MESSAGE IF OLD AND WRONG.
C*
C*     INPUT ARGUMENTS :
C*          CNAME  - COMMON BLOCK NAME
C*          ILEN   - THE LENGTH OF COMMON IN THIS SUBPROGRAM
C*
C*     COMMON BLOCKS :
C*          FILES, RUNOPT, COMMNS, SUBPRS, ONEVAR
C*
C*     FILE REFERENCES :
C*          NOUT
C*
C*     ERROR PROCESSING :
C*          CHECK THE NUMBER OF COMMONS AGAINST MAXIMUM
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          30-JAN-84     INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXSUB=400,MAXCOM=150,MREF=225,MXCHR=12)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / COMMNS / NAMES(MAXCOM), LENS(MAXCOM), ILENS(MAXCOM), 
     $                  NCERR, NUMCOM, NCOMMN
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      CHARACTER *(MXCHR) SUBNAM, SNAME, NAMES, CNAME, VNAME
      CHARACTER *150 OPSTRG     
      CHARACTER *100 CHART
      CHARACTER *15 SAVNAM
      CHARACTER *10 REFS
      CHARACTER *3 PRSECT
      CHARACTER *1 TYPE, CFLAG
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
C
      DO 10 I = 1, NUMCOM
         IF (CNAME .EQ. NAMES(I))GO TO 20
10       CONTINUE
C
C --- NEW NAME, ADD TO LIST AND MARK CHART
C
      IF (NUMCOM .EQ. MAXCOM) CALL FATAL
     $ ('TOO MANY COMMON BLOCKS IN SUBPROGRAM '//SNAME,MAXCOM)
      NUMCOM = NUMCOM + 1
      NAMES(NUMCOM) = CNAME
      LENS(NUMCOM) = ILEN
      IF ( MAPC )CHART(NUMSUB)(NUMCOM:NUMCOM) = 'X'
      RETURN
C
C --- COMMON NAME PREVIOUSLY FOUND
C
20    ILENS(NCOM) = ILEN
      IDLEN(NCOM) = LENS(I)
      IF (ILEN .NE. LENS(I)) NCERR = NCERR + 1
      IF ( MAPC )CHART(NUMSUB)(I:I) = 'X'
      RETURN
      END
C
C---END COMFND
C
      SUBROUTINE GETSUB
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETSUB           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET NEW SUBPROGRAM NAME
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF 94035              
C*          (415)965-5578                           
C*
C*     PURPOSE :
C*          EXTRACTS SUBPROGRAM NAME FROM MAP
C*
C*     METHODOLOGY :
C*          LOOKS FOR 'ENTRY POINTS' MESSAGE.
C*
C*     COMMON BLOCKS :
C*          COMMNS, FILES, FLIST, SUBPRS, RUNOPT,
C*          ONESUB, FLAGS
C*
C*     FILE REFERENCES :
C*          NOUT, NVFIL
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN, FATAL
C*
C*     ERROR PROCESSING :
C*          CHECK FOR MAXIMUM NUMBER OF SUBPROGRAMS
C*
C*     TRANSPORTABILITY LIMITATIONS :
C*          VAX FORTRAN MAP EXPECTED
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          18-NOV-87     MODIFIED TO SEARCH FOR FIELD HEADINGS
C*          30-JAN-84     INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXSUB=400,MAXVPS=1000,MAXCOM=150,MXCHR=12,MXCSUB=60)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / COMMNS / NAMES(MAXCOM), LENS(MAXCOM), ILENS(MAXCOM), 
     $                  NCERR, NUMCOM, NCOMMN
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / FLIST / LINE, EOF, FF
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / ONESUB / SNAMES(MAXVPS), STYPE(MAXVPS), SIZE(MAXVPS),
     $                  VCOMM(MAXVPS), SCOMMN(0:MXCSUB), NSVAR
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      CHARACTER *(MXCHR) SNAMES, SCOMMN, VCOMM, SUBNAM, SNAME, NAMES
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *100 CHART
      CHARACTER *15 SAVNAM
      CHARACTER *1 FF, STYPE
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT,
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF, COM, DONE, GO, DID, NML, BD
      INTEGER SIZE
C
C --- CHECK FOR BLOCK DATA
C
      IF (SCOMMN(0) .NE. '$CODE') THEN
         SNAME = 'BLOCKDATA'
         COM = .FALSE.    
         GO TO 25
      ENDIF
10    CALL GETLIN
      IEP = INDEX(LINE,'ENTRY POINTS')
      IF (IEP .EQ. 0) GO TO 10
15    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 15
      INAM = INDEX(LINE,'NAME')
20    CALL GETLIN
      IF (LINE .EQ. ' ')GO TO 20
      SNAME = LINE(INAM:INAM+MXCHR-1)
25    SUBNAM(NUMSUB) = SNAME
      IF (NUMSUB .EQ. MAXSUB) THEN
         CALL FATAL
     $ ('NUMBER OF SUBPROGRAMS EXCEEDS PROGRAM DIMENSIONS',MAXSUB)
      ELSE
C
C --- DON'T PROCESS 'Q1Q' ROUTINES INSERTED BY 'PREBUG'
C
         IF (SNAME(1:3) .NE. 'Q1Q') THEN
            NUMSUB = NUMSUB + 1
            WRITE(NOUT,900) SNAME
            WRITE(NVFIL,900) SNAME
         ELSE       
C
C --- EXIT TO EOF PROCESSING
C
             Q1Q = .TRUE.
             CALL GETLIN
         ENDIF
      ENDIF
C
C --- REPLACE ANY LOCAL SECTIONS WITH SUBPROGRAM NAME
C
      DO 30 I = 0, NCOMMN
         IF (SCOMMN(I)(1:1) .EQ. '$') 
     $       SCOMMN(I) = '$' // SNAME(1:LENGTH(SNAME))
30    CONTINUE
      RETURN
900   FORMAT(//,' SUBPROGRAM ',A)
      END                           
C
C---END GETSUB
C
      SUBROUTINE PROSUB
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          PROSUB           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          PROCESS SUBPROGRAM DATA                 
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          EXTRACT THE NAME OF THIS SUBPROGRAM AND PREPARE TO SEARCH   
C*          FOR VARIABLES.                                              
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FLIST, ONESUB, FLAGS, COMMNS, ONEVAR
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN, PROVAR, PROARR, DOLAB, DOPAR, MESSG
C*
C*     TRANSPORTABILITY LIMITATIONS :
C*          EXPECTS THE VAX FORTRAN 4.0 CROSS-REFERENCE LISTING FORMAT
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          07-OCT-86    MODIFIED FOR COMMON BLOCK VERIFICATION
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
C
C	PROCESS SUBPROGRAM DATA:
C
C	SKIP UNTIL COLUMN(1:3) NOT BLANK OR END-OF-PROGRAM
C	IF COLUMN(1:3) = 'VAR' THEN BEGIN
C		DO VARIABLES
C	        SKIP UNTIL COLUMN(1:3) NOT BLANK OR END-OF-PROGRAM
C		END
C	IF COLUMN(1:3) = 'ARR' THEN BEGIN
C		DO ARRAYS
C		SKIP UNTIL COLUMN(1:3) NOT BLANK OR END-OF-PROGRAM
C		END
C	IF COLUMN(1:3) = 'PAR' THEN BEGIN
C		DO PARAMETERS
C		SKIP UNTIL COLUMN(1:3) NOT BLANK OR END-OF-PROGRAM
C		END
C	IF COLUMN(1:3) = 'LAB' THEN DO LABELS
C	END
C
      PARAMETER (MAXVPS=1000,MXCHR=12,MXCSUB=60)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / FLIST / LINE, EOF, FF
      COMMON / ONESUB / SNAMES(MAXVPS), STYPE(MAXVPS), SIZE(MAXVPS),
     $                  VCOMM(MAXVPS), SCOMMN(0:MXCSUB), NSVAR
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      CHARACTER *(MXCHR) VCOMM, SCOMMN, SNAMES
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *15 SAVNAM
      CHARACTER *1 STYPE, FF
      INTEGER SIZE
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT,
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF, COM, DONE, DID, GO, BD, NML
C
      IF (NML) THEN
         IF (.NOT. NMLST) THEN
            WRITE (NVFIL,900)
            RETURN
         ELSE
            WRITE (NVFIL,910)
         ENDIF
      ENDIF
      NSVAR = 0
      DID = .FALSE.
      DONE = .FALSE.
C
C --- SKIP ENTRY POINTS (IF THERE WERE ANY)
C
      CALL GETLIN
C
C --- FIND NEXT SECTION
C
10    IF ( LINE(1:3) .EQ. '   ') THEN
         CALL GETLIN
         GO TO 10
      ENDIF
C
C --- VARIABLE SECTION
C
      IF (LINE(1:3) .EQ. 'VAR') THEN
         DID = .TRUE.
         CALL PROVAR 
20       IF (LINE(1:3) .EQ. '   ') THEN
            CALL GETLIN
            GO TO 20
         ENDIF
      ENDIF
C
C --- RECORD SECTION
C
      IF ((LINE(1:3) .EQ. 'REC') .AND. (LINE(8:10) .NE. 'ARR')) THEN
         IF (.NOT. DID) DID = .TRUE.
         CALL PROREC
25       IF (LINE(1:3) .EQ. '   ') THEN
            CALL GETLIN
            GO TO 25
         ENDIF
      ENDIF         
C
C --- ARRAY SECTION
C
      IF (LINE(1:3) .EQ. 'ARR') THEN
         IF (.NOT. DID) DID = .TRUE.
         CALL PROARR 
30       IF (LINE(1:3) .EQ. '   ') THEN
            CALL GETLIN
            GO TO 30
         ENDIF
      ENDIF
C
C --- RECORD ARRAY SECTION
C
      IF ((LINE(1:3) .EQ. 'REC') .AND. (LINE(8:10) .EQ. 'ARR')) THEN
         IF (.NOT. DID) DID = .TRUE.         
         CALL DORECA
35       IF (LINE(1:3) .EQ. '   ') THEN
            CALL GETLIN
            GO TO 35
         ENDIF
      ENDIF
      IF ( RUNVBL ) THEN
C
C --- PARAMETER SECTION
C
         IF (LINE(1:3) .EQ. 'PAR') THEN
            CALL DOPAR
50          IF (LINE(1:3) .EQ. '   ') THEN
               CALL GETLIN
               GO TO 50
            ENDIF
         ENDIF
C
C --- LABEL SECTION
C       
         IF (LINE(1:3) .EQ. 'LAB') CALL DOLAB
      ENDIF
      RETURN
900   FORMAT (/' ***** THIS ROUTINE CONTAINS NAMELIST STATEMENT(S).'/
     $        '       NO VARIABLE OR VERIFY CHECKING PERFORMED.')
910   FORMAT (/' ***** Warning: this subprogram contains a NAMELIST'
     $         ' statement! *****',/
     $         '       Warning/Error messages produced by VARIABLE and'
     $         ' VERIFY checking'/
     $         '       may not be accurate.'/)
      END
C
C---END PROSUB
C
      SUBROUTINE PROVAR 
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **         PROVAR            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          PROCESS VARIABLES
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA.   94035              
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          CONTROLS PROCESSING OF VARIABLE SECTION OF LISTING.
C*          
C*     METHODOLOGY : 
C*          BASED ON SUBPROGRAM DOVAR IN PROGRAM VARIABLE BY
C*          ART RAGOSTA  
C*
C*     COMMON BLOCKS :
C*          FLIST, RUNOPT, FLAGS
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN, CAPS, INDEX, GETVAR, CHKCOM, CHKVAR
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          NOVEMBER 18, 1987       MODIFIED TO SEARCH FOR FIELDS IN
C*                                   LISTING
C*          AUGUST 12, 1987         MODIFIED TO SKIP VARIABLE CHECKING
C*                                   FOR BLOCK DATA
C*          OCTOBER 8, 1986         MODIFIED FOR COMMON BLOCK VERIFICATION
C*          AUGUST 26, 1986         INITIAL VERSION 
C*           
C*
C***********************************************************************
C*
      COMMON / FLIST / LINE, EOF, FF
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *15 SAVNAM
      CHARACTER *1 FF
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT,
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF, COM, DONE, GO, DID, NML, BD
C
10    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 10
C
C --- LOCATE THE FIELDS
C
      ITYP  = INDEX(LINE,'TYPE') 
      INAM  = INDEX(LINE,'NAME')
      IATT  = INDEX(LINE,'ATTR')
      ISREF = INDEX(LINE,'REFER')
C
C --- SKIP DOWN TO DATA
C
15    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 15
C
C --- GET A VARIABLE
C
20    CALL GETVAR (ITYP, INAM, IATT, ISREF)
C
C --- CHECK IF IT IS USED
C
      IF ( RUNVFY .AND. COM .AND. GO ) CALL CHKCOM 
C
C --- DON'T CHECK VARAIBLES IN BLOCK DATA
C
      IF ( RUNVBL .AND. (.NOT. BD)) CALL CHKVAR
C
C --- SECTION ENDS IN EITHER TWO CONVSECUTIVE BLANK LINES OR 
C ---  NON-BLANK COLUMN 1
C
      IF (LINE(1:1) .EQ. ' ') THEN
         IF (LINE .NE. ' ') GO TO 20
         CALL GETLIN
         IF ((LINE .NE. ' ') .AND. (LINE(1:1) .EQ. ' ')) GO TO 20
      ENDIF
      RETURN
      END
C
C---END PROVAR
C  
      SUBROUTINE GETVAR (ITYP, INAM, IATT, ISREF)
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETVAR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET VARIABLE STUFF                      
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          EXTRACT THE RELEVANT INFORMATION ABOUT A SINGLE VARIABLE.   
C*
C*     INPUT ARGUMENTS :
C*          ITYP  -  COLUMN NUMBER OF 'TYPE' FIELD HEADING IN LISTING
C*          INAM  -     "     "    "  'NAME'  "      "      "   "
C*          IATT  -     "     "    "  'ATTRIBUTES' FIELD HEADING
C*          ISREF -     "     "    "  'REFERENCES'   "     "
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FLIST, SUBPRS, ONESUB, ONEVAR
C*
C*     SUBPROGRAM REFERENCES :
C*          RIGHT, FATAL, GETLIN
C*
C*     ERROR PROCESSING :
C*          CHECK THE NUMBER OF LINES REFERENCED
C*
C*     TRANSPORTABILITY LIMITATIONS :
C*          EXPECTS VAX FORTRAN 4.0 CROSS-REFERENCE LISTING
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          08-JUL-88    MODIFIED TO CALCULATE MAXIMUM NUMBER OF
C*                        REFERENCES TO BE FOUND ON LINE
C*          18-NOV-87    MODIFIED TO PARAMETERIZE COLUMN NUMBERS
C*          12-AUG-87    MODIFIED TO GET NUMERIC REFERENCES FOR
C*                       VARIABLE CHECKING
C*          08-OCT-86    MODIFIED FOR COMMON BLOCK VERIFICATION
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
C
C	GET VARIABLE STUFF:
C
C	COLUMNS(1:3) = PROGRAM SECTION
C	COLUMN(15)   = TYPE
C	COLUMN(17)   = SIZE
C	COLUMN(20:30) = NAME
C	COLUMN(34)   = COMMON FLAG
C	REPEAT
C		COLUMN(45:54,55:64,..115-124) = REFERENCES
C		UNTIL (NEXT LINE IS NEW VARIABLE
C		   OR (NEXT LINE IS BLANK
C		  AND  NOT END-OF-PAGE))
C	END
C
      PARAMETER (MAXSUB=400,MAXVPS=1000,MREF=225,MAXCOM=150,MXCHR=12,
     $           MXCSUB=60)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLIST / LINE, EOF, FF
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / ONESUB / SNAMES(MAXVPS), STYPE(MAXVPS), SIZE(MAXVPS),
     $                  VCOMM(MAXVPS), SCOMMN(0:MXCSUB), NSVAR
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      CHARACTER *(MXCHR) SUBNAM, SNAMES, VCOMM, SCOMMN, SNAME,
     $              VNAME
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *100 CHART
      CHARACTER *20 TMPNAM
      CHARACTER *15 SAVNAM, ADDRESS
      CHARACTER *10 REFS
      CHARACTER *6 TMPTYP
      CHARACTER *3 PRSECT, C
      CHARACTER *1 TYPE, CFLAG, STYPE, FF
      INTEGER TSIZE, SIZE
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF
C
C --- GET COMMON BLOCK NUMBER (IF ANY)
C
      ADDRESS = LINE(1:ITYP-1)
      ID = INDEX(ADDRESS,'-')
      PRSECT = ADDRESS(1:ID-1)
      READ(PRSECT,910,ERR=1)NSECT
      GO TO 2
1     NSECT = 0
C
C --- VARIABLE TYPE
C
2     IF ( RUNVBL ) THEN
         TMPTYP = LINE(ITYP:INAM-1)
         CALL LEFT (TMPTYP)
         IF (TMPTYP .EQ. 'CHAR') THEN
            TYPE = 'S'
            TSIZE = 0
         ELSE
            TYPE = TMPTYP(1:1)
            ISTAR = INDEX(TMPTYP,'*')
            C    =  TMPTYP(ISTAR+1:)
            CALL RIGHT(C)
            READ(C,910) TSIZE
         ENDIF
C
C --- VARIABLE NAME
C
         TMPNAM = LINE(INAM:IATT-1)
         VNAME = TMPNAM(1:MXCHR)
         NSVAR = NSVAR + 1
         IF (NSVAR .GT. MAXVPS) CALL FATAL
     $    ('Maximum number of variables exceeded for this subprogram.',
     $      MAXVPS)
         SNAMES(NSVAR) = VNAME
         SIZE(NSVAR)  = TSIZE
         STYPE(NSVAR) = TYPE
C
C --- COMMON BLOCK NAME OR LOCAL DATA SEGMENT
C 
         IF (SCOMMN(NSECT) .NE. '$CODE') THEN
            VCOMM(NSVAR) = SCOMMN(NSECT)
         ELSE
            VCOMM(NSVAR) = '$' // SUBNAM(NUMSUB)
         ENDIF
         CFLAG = LINE(IATT:IATT)
      ENDIF
C
C --- GET REFERENCES
C
      CALL GETREFS(ISREF,IATT)
C
      RETURN
910   FORMAT(I3)
      END
C
C---END GETVAR
C
      SUBROUTINE CHKCOM 
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          CHKCOM           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          CHECK COMMON
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA.   94035              
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          COMPARE COMMON BLOCK NUMBER OF THIS VARIABLE TO KNOWN
C*          COMMON BLOCK NUMBERS; IF KNOWN, CHECK LAST REFERENCE
C*          TO DETERMINE IF THIS COMMON BLOCK IS USED.
C*          
C*     COMMON BLOCKS :
C*          VERCOM, FORSTM, ONEVAR, FLAGS
C*
C*     SUBPROGRAM REFERENCES :
C*          RIGHT
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          AUGUST 26, 1986         INITIAL VERSION 
C*           
C*
C***********************************************************************
C*
      PARAMETER (MAXCOM=150,MREF=225,MXCHR=12)
      COMMON / VERCOM / CNAMES(MAXCOM), NUMS(MAXCOM), NTCOM, NVFERR
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      COMMON / FORSTM / STMT, CARD, LINUM, NUMSTM, CONT, NSTART, NCOLST
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      CHARACTER *(MXCHR) CNAMES, VNAME
      CHARACTER *3000 STMT
      CHARACTER *80 CARD
      CHARACTER *10 REFS
      CHARACTER *3 PRSECT
      CHARACTER *1 TYPE, CFLAG
      LOGICAL CONT, KNOWN, COM, DONE, GO, DID, NML, BD
C
C --- CHECK IF PROGRAM SECTION FOR THIS VARIABLE IS A KNOWN
C ---  COMMON BLOCK NUMBER
C
      KNOWN = .FALSE. 
      DO 10 I = 1,NCOM
         IF ( NSECT .EQ. NUMS(I) ) THEN
            INDX = I
            KNOWN = .TRUE.
            GO TO 20
         ENDIF
10    CONTINUE
C
C --- IF KNOWN, CHECK LAST REFERENCE TO THIS VARIABLE
C
20    IF (KNOWN) THEN
         IF ( (NREFS(NREF) .GE. NSTART) .OR.
     $        (INDEX (REFS(NREF),'D') .NE. 0) ) THEN  
C
C --- COMMON BLOCK IS USED, DELETE IT FROM LIST TO BE CHECKED
C
            NUMS(INDX) = -1
            NTCOM = NTCOM - 1
            IF (NTCOM .EQ. 0) DONE = .TRUE.
         ENDIF
      ENDIF
      RETURN
      END
C
C---END CHKCOM
C
      SUBROUTINE CHKVAR
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          CHKVAR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          CHECK VARIABLE                          
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          CHECK THE PRESENT VARIABLE FOR INITIALIZATION AND USE.      
C*
C*     COMMON BLOCKS :
C*          FILES, RUNOPT, ONEVAR, FORSTM
C*
C*     FILE REFERENCES :
C*          NVFIL
C*
C*     SUBPROGRAM REFERENCES :
C*          RIGHT, INDEX
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          ROUTINE RIGHT IS IN 'MERLIB'
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          10-AUG-87    FIXED '1ST REFERENCE IS NOT INITIALIZATION'
C*                       CHECKING
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
C
C	CHECK VARIABLE:
C
C	IF NOT IN COMMON THEN BEGIN
C		IF ONLY ONE REFERENCE THEN ERROR'VAR ONLY REF ONCE'
C		IF NOT PARAMETER TO ROUTINE THEN BEGIN
C			IF FIRST REFERENCE NOT ARG OR = THEN BEGIN
C                           CHECK FOR SPECIFICATION
C                           IF DOESN'T NEED TO BE TYPED THEN WARN'MAY BE
C							UNINITIALIZED'
C                           END
C			IF NO REFERENCE IS ARG OR = THEN WARN'MAY BE
C							UNREFERENCED'
C			IF ALL REFERENCES = THEN ERROR'UNUSED CALCULATED VAR'
C			END
C		END
C	END
C
      PARAMETER (MREF=225, MAXCOM=150, MXCHR=12)
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      COMMON / FORSTM / STMT, CARD, LINUM, NUMSTM, CONT, NSTART, NCOLST
      CHARACTER *(MXCHR) VNAME
      CHARACTER *3000 STMT
      CHARACTER *80 CARD
      CHARACTER *150 OPSTRG
      CHARACTER *10 REFS, WREF 
      CHARACTER *15 SAVNAM
      CHARACTER *3 PRSECT
      CHARACTER *1 TYPE, CFLAG
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, CONT, UCHECK, NMLST
      LOGICAL ALL, NONE, DINIT
C
C --- NO CHECKING IF IN COMMON
C
      IF (CFLAG .EQ. 'C') RETURN
C
C ----- ONE REFERENCE IS ALWAYS AN ERROR
C
      IF (NREF .EQ. 1) THEN
C
C -------- HOWEVER, IT MIGHT BE AN IMPLIED DO...  ( '10(2)=' )
C
         WREF = REFS(1)
         CALL RIGHT(WREF)
         IF (WREF(9:9) .NE. ')') THEN
            WRITE(NVFIL,900) VNAME(1:LENGTH(VNAME))
            NVERR = NVERR + 1
         ELSE
            WRITE(NVFIL,905) VNAME(1:LENGTH(VNAME))
         ENDIF
         RETURN
      ENDIF
C
C ------ NO OTHER ERRORS FOR PARAMETERS TO THIS SUBPROGRAM
C
      IF (PRSECT .EQ. ' AP') RETURN
      ALL  = .TRUE.
      NONE = .TRUE.
      DINIT = .FALSE.
      INDX = 1
5     IF (NREFS(INDX) .LT. NSTART) THEN
C
C --- CHECK FOR DATA STATEMENT INITIALIZATION IN NON-EXECUTABLE CODE
C
         IF (INDEX(REFS(INDX),'D') .NE. 0) THEN
            DINIT = .TRUE.
            NONE = .FALSE.
         ENDIF
         INDX = INDX + 1
         IF (INDX .LE. NREF) GO TO 5
      ENDIF
      IF (INDX .GT. NREF) THEN
C
C --- ALL REFERENCES TO THIS VARIABLE WERE IN NON-EXECUTABLE CODE,
C --- THEREFORE VARIABLE IS UNUSED
C
         WRITE(NVFIL,920) VNAME(1:LENGTH(VNAME))
         NVERR = NVERR + 1
         RETURN
      ELSE
C
C --- CHECK FIRST REFERENCE IN EXECUTABLE CODE
C
         IF ( LFIRST ) THEN
            IF ((INDEX(REFS(INDX),'A') .EQ. 0) .AND.
     $          (INDEX(REFS(INDX),'=') .EQ. 0) .AND.
     $          (.NOT. DINIT))
     $          WRITE(NVFIL,910) VNAME(1:LENGTH(VNAME))
         ENDIF
C
C --- CHECK REST OF REFERENCES
C
         DO 10 I = INDX, NREF
            WREF = REFS(I)
            CALL RIGHT(WREF)
C
C ------------ EITHER ALL REFERENCES WERE INITIALIZATION OR NONE WERE
C
            IF ((WREF(10:10) .EQ. 'A') .OR. (WREF(10:10) .EQ. '=')) 
     $          NONE = .FALSE.
            IF (WREF(10:10) .NE. '=') ALL = .FALSE.
10       CONTINUE
      ENDIF
      IF ( ALL ) THEN
C
C --- UNUSED VARIABLE
C
         WRITE(NVFIL,920) VNAME(1:LENGTH(VNAME))
         NVERR = NVERR + 1 
      ENDIF
      IF ( NONE ) THEN
C
C --- UNINITIALIZED VARIABLE
C
         WRITE(NVFIL,930) VNAME(1:LENGTH(VNAME))
         NVERR = NVERR + 1
      ENDIF
      RETURN
900   FORMAT('--- Error, "',A,'" is referenced only once.')
905   FORMAT('-- Warning, "',A,'" appears only in one statement.')
910   FORMAT('-- Warning, the first reference to "',A,'" is not an',
     $  ' initialization.')
920   FORMAT('--- Error, "',A,'" appears to be unused.')
930   FORMAT('--- Error, "',A,'" appears to be uninitialized.')
      END
C
C---END CHKVAR
C
      SUBROUTINE PROREC
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          PROREC           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     AUTHOR :
C*          L JURGELEIT
C*          MS 219-3
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA  94035
C*          (415) 604-5578
C*
C*     DESCRIPTION :
C*          CONTROLS PROCESSING OF RECORD SECTION OF LISTING
C*
C*     COMMON BLOCKS :
C*          FLIST, RUNOPT, FLAGS
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN, GETREC, CHKCOM, CHKVAR
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          27-JUN-1990  -  INITIAL VERSION
C*
C***********************************************************************
C*
      COMMON / FLIST / LINE, EOF, FF
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *15 SAVNAM
      CHARACTER *1 FF
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT,
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF, COM, DONE, GO, DID, NML, BD
C
10    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 10
C
C --- LOCATE THE FIELDS
C
      INAM = INDEX(LINE,'NAME')
      ISTR = INDEX(LINE,'STRUCTURE')
      IATT = INDEX(LINE,'ATTR')
      IBYT = INDEX(LINE,'BYTES')
      ISREF= INDEX(LINE,'REFER') 
C
C --- SKIP DOWN TO DATA
C
15    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 15
C
C --- GET A RECORD
C
20    CALL GETREC (INAM, ISTR, IATT, IBYT, ISREF)
C
C --- VERIFY CHECKING
C
      IF ( RUNVFY .AND. COM .AND. GO ) CALL CHKCOM      
C
C --- CHECK IF RECORD IS USED
C
      IF ( RUNVBL .AND. (.NOT. BD)) CALL CHKVAR
C
C --- SECTION ENDS IN EITHER TWO CONSECUTIVE BLANK LINES OR
C --- NON-BLANK COLUMN 1
C
      IF (LINE(1:1) .EQ. ' ') THEN
         IF (LINE .NE. ' ') GO TO 20
         CALL GETLIN
         IF ((LINE .NE. ' ') .AND. (LINE(1:1) .EQ. ' ')) GO TO 20
      ENDIF
      RETURN
      END
C
C---END PROREC
C
      SUBROUTINE GETREC (INAM, ISTR, IATT, IBYT, ISREF)
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETREC           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     AUTHOR :
C*          L JURGELEIT
C*          MS 219-3
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA  94035
C*          (415) 604-5578
C*
C*     DESCRIPTION :
C*          EXTRACT RELEVANT INFORMATION ABOUT A SINGLE RECORD
C*
C*     INPUT ARGUMENTS :
C*          INAM - COLUMN NUMBER OF 'NAME' FIELD HEADING IN LISTING
C*          ISTR -    "     "    "  'STRUCTURE' FIELD HEADING
C*          IATT -    "     "    "  'ATTRIBUTE'   "     "
C*          IBYT -    "     "    "  'BYTE'        "     "
C*          ISREF-    "     "    "  'REFERENCES'  "     "
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FLIST, SUBPRS, ONESUB, ONEVAR
C*
C*     SUBPROGRAM REFERENCES :
C*          RIGHT, FATAL, GETLIN
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          REQUIRES ARTA SYSTEM LIBRARY 'MERLIB'
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          29-JUN-1990  -  INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXSUB=400,MAXVPS=1000,MREF=225,MAXCOM=150,MXCHR=12,
     $           MXCSUB=60)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLIST / LINE, EOF, FF
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / ONESUB / SNAMES(MAXVPS), STYPE(MAXVPS), SIZE(MAXVPS),
     $                  VCOMM(MAXVPS), SCOMMN(0:MXCSUB), NSVAR
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      CHARACTER *(MXCHR) SUBNAM, SNAMES, VCOMM, SCOMMN, SNAME,
     $              VNAME
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *100 CHART
      CHARACTER *20 TMPNAM
      CHARACTER *15 SAVNAM, ADDRESS
      CHARACTER *10 REFS
      CHARACTER *8 BYTES
      CHARACTER *3 PRSECT
      CHARACTER *1 TYPE, CFLAG, STYPE, FF
      INTEGER SIZE
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF
C
C --- GET COMMON BLOCK NUMBER (IF ANY)
C
      ADDRESS = LINE(1:INAM-1)
      ID = INDEX(ADDRESS,'-')
      PRSECT = ADDRESS(1:ID-1)      
      READ (PRSECT,910,ERR=1)NSECT
      GO TO 2
1     NSECT = 0
2     IF ( RUNVBL ) THEN
C
C --- VARIABLE NAME
C
         TMPNAM = LINE(INAM:ISTR-1)
         VNAME = TMPNAM(1:MXCHR)
         NSVAR = NSVAR + 1
         IF (NSVAR .GT. MAXVPS) CALL FATAL
     $    ('Maximum number of variables exceeded for this subprogram.',
     $      MAXVPS)
         SNAMES(NSVAR) = VNAME
C
C --- VARIABLE SIZE
C
         BYTES = LINE(IBYT-3:IBYT+4)                  
         READ(BYTES,900,ERR=3) IDIM
         GO TO 4
3        IDIM = 0
4        SIZE(NSVAR) = IDIM
C
C --- VARIABLE TYPE - RECORD - VAX SPECIFIC
C
         STYPE(NSVAR) = 'V'
C
C --- COMMON BLOCK OR LOCAL DATA SEGMENT
C
         IF (SCOMMN(NSECT) .NE. '$CODE') THEN
            VCOMM(NSVAR) = SCOMMN(NSECT)
         ELSE
            VCOMM(NSVAR) = '$' // SUBNAM(NUMSUB)
         ENDIF
C
         CFLAG = LINE(IATT:IATT)
      ENDIF
C
C --- GET REFERENCES
C
      CALL GETREFS (ISREF, IATT)
C
      RETURN
900   FORMAT(I8)
910   FORMAT(I3)
      END
C
C---END GETREC
C
      SUBROUTINE PROARR 
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          PROARR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          PROCESS ARRAY
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA.   94035              
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          PROCESS THE 'ARRAYS' SECTION OF THE LISTING.
C*          
C*     METHODOLOGY :
C*          BASED ON SUBPROGRAM DOARR IN PROGRAM VARIABLE
C*          BY ART RAGOSTA
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FLIST, FLAGS
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN, GETARR, CHKCOM, CHKVAR
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          NOVEMBER 18, 1987       MODIFIED TO SEARCH FOR FIELDS
C*          OCTOBER 2, 1986         MODIFIED TO INCLUDE VARIABLE 
C*                                  CHECKING  
C*          AUGUST 26, 1986         INITIAL VERSION 
C*
C***********************************************************************
C*
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLIST / LINE, EOF, FF
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *15 SAVNAM
      CHARACTER *1 FF
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF, DONE, COM, GO, DID, NML, BD
C                         
10    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 10
C
C --- LOCATE THE DATA FIELDS
C
      ITYP  = INDEX(LINE,'TYPE') 
      INAM  = INDEX(LINE,'NAME')
      IATT  = INDEX(LINE,'ATTR')
      IBYT  = INDEX(LINE,'BYTES')
      ISREF = INDEX(LINE,'REFER')
C
C --- SKIP DOWN TO DATA
C
15    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 15
C
C --- GET AN ARRAY 
C
20    CALL GETARR (ITYP, INAM, IATT, ISREF, IBYT)
C
C --- CHECK FOR USE
C
      IF ( RUNVFY .AND. COM .AND. GO ) CALL CHKCOM 
C
C --- DON'T CHECK ARRAYS IN BLOCK DATA
C
      IF ( RUNVBL .AND. (.NOT. BD)) CALL CHKVAR
C
C --- SECTION ENDS IN EITHER TWO CONSECUTIVE BLANK LINES OR A 
C --- NON-BLANK COLUMN 1
C
      IF (LINE(1:1) .EQ. ' ') THEN
         IF (LINE .NE. ' ') GO TO 20
         CALL GETLIN
         IF ((LINE .NE. ' ') .AND. (LINE(1:1) .EQ. ' ')) GO TO 20
      ENDIF
      RETURN
      END
C
C---END PROARR
C
      SUBROUTINE GETARR (ITYP, INAM, IATT, ISREF, IBYT)
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETARR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          GET ARRAY STUFF                         
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          EXTRACT RELEVANT INFORMATION FOR ONE ARRAY.                 
C*
C*     INPUT ARGUMENTS
C*          ITYP  -  COLUMN NUMBER OF 'TYPE' FIELD IN ARRAY SECTION OF LISTING
C*          INAM  -  COLUMN NUMBER OF 'NAME' FIELD
C*          IATT  -  COLUMN NUMBER OF 'ATTRIBUTES' FIELD
C*          ISREF -  COLUMN NUMBER OF 'REFERENCES' FIELD
C*          IBYT  -  COLUMN NUMBER OF 'BYTES' FIELD
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FLIST, SUBPRS, ONESUB, ONEVAR
C*
C*     SUBPROGRAM REFERENCES :
C*          RIGHT, FATAL, GETLIN
C*
C*     ERROR PROCESSING :
C*          CHECK FOR MAXIMUM NUMBER OF REFERENCES TO THIS ARRAY
C*
C*     TRANSPORTABILITY LIMITATIONS :
C*          EXPECTS VAX FORTRAN 4.0 CROSS-REFERENCE LISTING
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          08-JUL-88    MODIFIED TO CALCULATE MAX NUMBER OF REFERENCES
C*                        POSSIBLE ON A LISTING LINE
C*          11-NOV-87    COLUMN NUMBERS PARAMETERIZED
C*          12-AUG-87    MODIFIED TO GET NUMERIC REFERENCES FOR
C*                       VARIABLE CHECKING
C*          08-OCT-86    MODIFIED FOR COMMON BLOCK VERIFICATION
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
C
C	GET ARRAY STUFF:
C
C	COLUMNS(1:3) = PROGRAM SECTION
C	COLUMN(15) = TYPE
C	COLUMN(20:30) = NAME
C	COLUMN(34) = COMMON FLAG
C	COLUMN(48:55) = SIZE
C	REPEAT
C		COLUMN(88-97,98-107,..118-127) = REFERENCES
C		UNTIL (NEXT LINE IS NEW VARIABLE
C		   OR (NEXT LINE IS BLANK
C		  AND  NOT END-OF-PAGE))
C	END
C
      PARAMETER (MAXSUB=400,MAXVPS=1000,MREF=225,MAXCOM=150,MXCHR=12,
     $           MXCSUB=60)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLIST / LINE, EOF, FF
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / ONESUB / SNAMES(MAXVPS), STYPE(MAXVPS), SIZE(MAXVPS),
     $                  VCOMM(MAXVPS), SCOMMN(0:MXCSUB), NSVAR
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      CHARACTER *(MXCHR) SUBNAM, SNAMES, VCOMM, SCOMMN, SNAME,
     $              VNAME
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE 
      CHARACTER *100 CHART
      CHARACTER *20 TMPNAM
      CHARACTER *15 SAVNAM, ADDRESS
      CHARACTER *10 REFS
      CHARACTER *8 C
      CHARACTER *6 TMPTYP
      CHARACTER *3 PRSECT, C1
      CHARACTER *1 STYPE, TYPE, CFLAG, FF
      INTEGER SIZE, TSIZE
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF
C
C --- GET COMMON BLOCK NUMBER (IF ANY)
C
      ADDRESS = LINE(1:ITYP-1)
      ID = INDEX(ADDRESS,'-')
      PRSECT = ADDRESS(1:ID-1)
      READ(PRSECT,910,ERR=1)NSECT
      GO TO 2
1     NSECT = 0
2     IF ( RUNVBL ) THEN
C
C --- VARIABLE TYPE
C
         TMPTYP = LINE(ITYP:INAM-1)
         CALL LEFT (TMPTYP)
         IF (TMPTYP .EQ. 'CHAR') THEN
            TYPE = 'S'
            TSIZE = 0
         ELSE
            TYPE = TMPTYP(1:1)
C           C1   = LINE(17:19)
C           CALL RIGHT(C1)
C           READ(C1,910) TSIZE
         ENDIF
C
C --- VARIABLE NAME
C
         TMPNAM = LINE(INAM:IATT-1)
         VNAME = TMPNAM(1:MXCHR)
         C     = LINE(IBYT-4:IBYT+3)
         NSVAR = NSVAR + 1
         IF (NSVAR .GT. MAXVPS) CALL FATAL
     $    ('Maximum number of variables exceeded for this subprogram.',
     $      MAXVPS)
         SNAMES(NSVAR) = VNAME
         READ(C,900,ERR=3) IDIM
         GO TO 4
C
C --- VARIABLE LENGTH ARRAYS
C
3        IDIM = 0
4        SIZE(NSVAR)  = IDIM
         STYPE(NSVAR) = TYPE
C
C --- COMMON BLOCK NAME OR LOCAL DATA SEGMENT
C
         IF (SCOMMN(NSECT) .NE. '$CODE') THEN
            VCOMM(NSVAR) = SCOMMN(NSECT)
         ELSE
            VCOMM(NSVAR) = '$' // SUBNAM(NUMSUB)
         ENDIF
C
         CFLAG = LINE(IATT:IATT)
      ENDIF
C
C --- GET REFERENCES
C
      CALL GETREFS(ISREF, IATT)
C
      RETURN
900   FORMAT(I8)
910   FORMAT(I3)
      END
C
C---END GETARR
C
      SUBROUTINE DORECA
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          DORECA           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     AUTHOR :
C*          L JURGELEIT
C*          MS 219-3
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA  94035
C*          (415) 604-5578
C*
C*     DESCRIPTION :
C*          PROCESS "RECORD ARRAYS" SECTION OF LISTING
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FLIST, FLAGS
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN, GETRECA, CHKCOM, CHKVAR
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*           2-JUL-1990  -  INITIAL VERSION
C*
C***********************************************************************
C*
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLIST / LINE, EOF, FF
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE
      CHARACTER *15 SAVNAM
      CHARACTER *1 FF
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF, DONE, COM, GO, DID, NML, BD
C                         
10    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 10
C
C --- LOCATE THE DATA FIELDS
C
      INAM = INDEX (LINE,'NAME')
      ISTR = INDEX (LINE,'STRU')
      IATT = INDEX (LINE,'ATTR')
      IBYT = INDEX (LINE,'BYTE')
      ISREF = INDEX (LINE,'REFER') 
C
C --- SKIP DOWN TO DATA
C
15    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 15
C
C --- GET A RECORD ARRAY 
C
20    CALL GETRECA (INAM, ISTR, IATT, IBYT, ISREF)      
C
C --- CHECK COMMON BLOCKS FOR USE
C
      IF (RUNVFY .AND. COM .AND. GO) CALL CHKCOM
C
C --- CHECK RECORD ARRAY FOR USE
C
      IF (RUNVBL .AND. (.NOT. BD)) CALL CHKVAR
C
C --- SECTION ENDS IN EITHER TWO CONSECUTIVE BLANK LINES OR A 
C --- NON-BLANK COLUMN 1
C
      IF (LINE(1:1) .EQ. ' ') THEN
         IF (LINE .NE. ' ') GO TO 20
         CALL GETLIN
         IF ((LINE .NE. ' ') .AND. (LINE(1:1) .EQ. ' ')) GO TO 20
      ENDIF
      RETURN
      END
C
C---END DORECA
C
      SUBROUTINE GETRECA (INAM, ISTR, IATT, IBYT, ISREF)
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          GETRECA          **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     AUTHOR :
C*          L JURGELEIT
C*          MS 219-3
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA  94035
C*          (415) 604-5578
C*
C*     DESCRIPTION :
C*          EXTRACT RELEVANT INFORMATION FOR ONE RECORD ARRAY.
C*
C*     INPUT ARGUMENTS :
C*          INAM  -  COLUMN NUMBER OF 'NAME' FIELD IN LIST FILE
C*          ISTR  -  COLUMN NUMBER OF 'STRUCTURE' FIELD
C*          IATT  -  COLUMN NUMBER OF 'ATTRIBUTES' FIELD
C*          IBYT  -  COLUMN NUMBER OF 'BYTES' FIELD
C*          ISREF -  COLUMN NUMBER OF 'REFERENCES' FIELD
C*
C*     COMMON BLOCKS :
C*           RUNOPT, FLIST, SUBPRS, ONESUB, ONEVAR
C*
C*     SUBPROGRAM REFERENCES :
C*           FATAL, FIRST, GETLIN
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*           REQUIRES ARTA SYSTEM LIBRARY "MERLIB"
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*           2-JUL-1990  -  INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXSUB=400,MAXVPS=1000,MREF=225,MAXCOM=150,MXCHR=12,
     $           MXCSUB=60)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FLIST / LINE, EOF, FF
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      COMMON / ONESUB / SNAMES(MAXVPS), STYPE(MAXVPS), SIZE(MAXVPS),
     $                  VCOMM(MAXVPS), SCOMMN(0:MXCSUB), NSVAR
      CHARACTER *(MXCHR) SUBNAM, SNAMES, VCOMM, SCOMMN, SNAME,
     $              VNAME
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE 
      CHARACTER *100 CHART
      CHARACTER *20 TMPNAM
      CHARACTER *15 SAVNAM, ADDRESS
      CHARACTER *10 REFS, C
      CHARACTER *3 PRSECT
      CHARACTER *1 STYPE, TYPE, CFLAG, FF
      INTEGER SIZE
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL EOF
C
C --- COMMON BLOCK NUMBER
C
      ADDRESS = LINE(1:INAM-1)
      ID = INDEX(ADDRESS,'-')
      PRSECT = ADDRESS(1:ID-1)
      READ(PRSECT,900,ERR=1) NSECT
      GO TO 2
1     NSECT = 0
2     IF (RUNVBL) THEN
C
C --- VARIABLE NAME
C
         TMPNAM = LINE(INAM:ISTR-1)
         VNAME = TMPNAM(1:MXCHR)
         IF (NSVAR .GT. MAXVPS) CALL FATAL
     $    ('Maximum number of variables exceeded for this subprogram.',
     $      MAXVPS)
         SNAMES(NSVAR) = VNAME
C
C --- VARIABLE TYPE - RECORD ARRAY - VAX SPECIFIC
C
         STYPE(NSVAR) = 'V'
C
C --- VARIABLE SIZE
C
         C = LINE(IATT+11:IBYT+4)
         READ(C,910,ERR=3) IDIM
         GO TO 4
C
C --- VARIABLE LENGTH ARRAYS
C
3        IDIM = 0
4        SIZE(NSVAR) = IDIM
C
C --- COMMON BLOCK NAME OR LOCAL DATA SEGMENT
C
         IF (SCOMMN(NSECT) .NE. '$CODE') THEN
            VCOMM(NSVAR) = SCOMMN(NSECT)
         ELSE
            VCOMM(NSVAR) = '$' // SUBNAM(NUMSUB)
         ENDIF
C
         CFLAG = LINE(IATT:IATT)
      ENDIF
C
C --- GET REFERENCES
C
      CALL GETREFS (ISREF, IATT)
C
      RETURN
900   FORMAT(I3)
910   FORMAT(I10)
      END
C
C---END GETRECA
C
      SUBROUTINE GETREFS (ISTART, IW)
C
      PARAMETER (MAXCOM=150, MXCHR=12, MREF=225)
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      COMMON / FLIST / LINE, EOF, FF
      CHARACTER *(MXCHR) VNAME
      CHARACTER *132 LINE
      CHARACTER *70 REFSTR
      CHARACTER *10 REFS, RREFS(10)
      CHARACTER *3 PRSECT
      CHARACTER *1 FF, TYPE, CFLAG, CH
      DIMENSION NRREFS(10)
      LOGICAL EOF
C
      NREF = 0
      REFSTR = LINE(ISTART:132)
      CALL LEFT (REFSTR)
      CALL RPARSE (REFSTR, REFS, NREFS, NREF)
10    CALL GETLIN
      CALL FIRST (LINE,CH,IREF)
      IF (IREF .GT. IW) THEN
         REFSTR = LINE(IREF:132)
         CALL RPARSE (REFSTR, RREFS, NRREFS, NRREF)
         DO 11 I = 1, NRREF
            REFS(NREF+I) = RREFS(I)
            NREFS(NREF+I) = NRREFS(I)
11       CONTINUE
         NREF = NREF + NRREF
         GO TO 10
      ENDIF
      RETURN
      END
C
      SUBROUTINE RPARSE (REFSTR, REFERS, NREFERS, NREFER)
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          RPARSE           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     AUTHOR :
C*          L JURGELEIT
C*          MS 219-3
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA  94035
C*          (415) 604-5578
C*
C*     DESCRIPTION :
C*          PARSE A "REFERENCE STRING" INTO INDIVIDUAL REFERENCES
C*
C*     INPUT ARGUMENTS :
C*          REFSTR - STRING CONTAINING REFERENCES 
C*                   (E.G., "93       95A       98(2)" ETC.)   
C*
C*     OUTPUT ARGUMENTS :
C*          REFERS - ARRAY CONTAINING INDIVIDUAL REFERENCES
C*          NREFER - NUMBER OF ENTRIES IN REFERS
C*          NREFERS - ARRAY OF NUMERICAL REFERENCES
C*
C*     SUBPROGRAM REFERENCES :
C*          ISDIGIT, RIGHT
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          REQUIRES ARTA SYSTEM LIBRARY "MERLIB"
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          18-JUL-1990  -  INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER(MXREF=15)
      CHARACTER *(*) REFSTR
      CHARACTER *10 REFERS(MXREF), TREF, TNREF
      DIMENSION NREFERS(MXREF)
C
      MXR = LENGTH(REFSTR)
      NREFER = 0
      IREF = 0
      ITR  = 0
      ITNR = 0
      TREF = ' '
      TNREF= ' '
10    IREF = IREF + 1
      IF (IREF .LE. MXR) THEN
         IF (REFSTR(IREF:IREF) .NE. ' ') THEN
C
C --- PACK REFERENCE
C
            ITR = ITR + 1
            TREF(ITR:ITR) = REFSTR(IREF:IREF)
            IF (ISDIGIT(REFSTR(IREF:IREF))) THEN
               ITNR = ITNR + 1
               TNREF(ITNR:ITNR) = REFSTR(IREF:IREF)
            ENDIF
         ELSE
C
C --- STORE REFERENCE
C
            NREFER = NREFER + 1
            REFERS(NREFER) = TREF
            CALL RIGHT (TNREF)
            READ (TNREF,900) LREF
            NREFERS(NREFER) = LREF            
            ITR = 0
            ITNR = 0
            TREF = ' '
            TNREF = ' '
C
C --- SKIP BLANKS TO NEXT REFERENCE
C
20          IREF = IREF + 1
            IF (REFSTR(IREF:IREF) .EQ. ' ') GO TO 20
            IREF = IREF - 1
         ENDIF
         GO TO 10
      ENDIF
C
C --- GET LAST REFERENCE
C
      NREFER = NREFER + 1
      REFERS(NREFER) = TREF
      CALL RIGHT(TNREF)
      READ (TNREF,900) LREF
      NREFERS(NREFER) = LREF
      RETURN
900   FORMAT(I10)
      END
C
C---END RPARSE
C
      SUBROUTINE DOPAR
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          DOPAR            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DO PARAMETERS               
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          PROCESS THE 'PARAMETERS' SECTION OF THE LISTING.
C*
C*     COMMON BLOCKS :
C*          FILES, FLIST
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          18-NOV-87    COLUMN NUMBERS PARAMETERIZED
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
C
C	DO PARAMETERS:
C
C	SKIP 3 LINES
C	REPEAT
C		GET REFERENCES ONLY
C		IF REFERENCES<2 THEN ERROR'UNUSED PARAMETER'
C		UNTIL END-OF-PARAMS
C
      PARAMETER (MREF=225, MXCHR=12, MAXCOM=150)
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / FLIST / LINE, EOF, FF
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      CHARACTER *(MXCHR) NAME, VNAME
      CHARACTER *132 LINE
      CHARACTER *10 REFS 
      CHARACTER *3 PRSECT
      CHARACTER *1 FF, TYPE, CFLAG
      LOGICAL EOF
C
C --- SKIP BLANK LINES
C
10    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 10
C
C --- FIND IN WHAT COLUMN REFERENCES BEGIN
C
      IREF = INDEX(LINE,'REFER')
C
C --- SKIP FIRST REFERENCE
C
      IREF = IREF + 10
      INAME = INDEX(LINE,'NAME')
15    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 15
C
C --- GET NAME OF THIS PARAMETER
C
20    NAME = LINE(INAME:INAME+MXCHR-1)
C
C --- ONLY ONE REFERENCE IS AN ERROR
C
      IF (LINE(IREF:IREF+9) .EQ. ' ') THEN
         WRITE (NVFIL,900) NAME
         NVERR = NVERR + 1
      ENDIF
C
C --- A SECTION ENDS IN EITHER TWO CONSECUTIVE BLANK LINES OR A NEW
C ---  HEADER (NON-BLANK COLUMN 1 )
C
      CALL GETLIN
30    IF ((LINE .NE. ' ') .AND. (LINE(1:1) .EQ. ' ')) THEN
         IF (LINE(INAME:INAME+MXCHR-1) .EQ. ' ') THEN    ! CONTINUATION
            CALL GETLIN
            GO TO 30
         ENDIF
         GO TO 20                          ! ELSE NEW LABEL
      ENDIF
      RETURN
900   FORMAT('--- Error, Parameter "',A,'" is only referenced once.')
      END
C
C---END DOPAR
C
      SUBROUTINE DOLAB
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          DOLAB            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DO LABELS                               
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          PROCESS THE 'LABELS' SECTION OF THE LISTING.                
C*
C*     COMMON BLOCKS :
C*          FILES, FLIST
C*
C*     SUBPROGRAM REFERENCES :
C*          GETLIN, CAPS
C*
C*     FILE REFERENCES :
C*          NVFIL
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          18-NOV-87    COLUMN NUMBERS PARAMETERIZED
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
C
C	DO LABELS:
C
C	SKIP 3 LINES
C	REPEAT
C		GET REFERENCES ONLY
C		IF REFERENCES<2 THEN ERROR'UNREFERENCED LABEL'
C		UNTIL END-OF-LABELS
C
      PARAMETER (MREF=225, MXCHR=12, MAXCOM=150)
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / FLIST / LINE, EOF, FF
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      CHARACTER *(MXCHR) VNAME
      CHARACTER *132 LINE
      CHARACTER *10 REFS 
      CHARACTER *7 NAME
      CHARACTER *3 PRSECT
      CHARACTER *1 TYPE, CFLAG, FF
      LOGICAL EOF
C
10    CALL GETLIN
      ILAB = INDEX (LINE,'LABEL')
      IF (ILAB .EQ. 0) GO TO 10
C
C --- FIND COLUMN WITH THE REFERENCES
C
      IREF = INDEX(LINE,'REFER')
C
C --- SKIP FIRST REFERENCE SINCE WE ARE CHECKING FOR MORE THAN ONE
C
      IREF = IREF + 10
C
C --- GO TO NEXT NON-BLANK LINE
C
15    CALL GETLIN
      IF (LINE .EQ. ' ') GO TO 15
C
C --- GET LABEL NUMBER
C
20    NAME  = LINE(ILAB:ILAB+6)
C
C --- ONLY ONE REFERENCE IS AN ERROR
C
      IF (LINE(IREF:IREF+10) .EQ. ' ') THEN
         I = INDEX (NAME,'''') - 1
         IF (I .LT. 0) THEN
            I = LENGTH(NAME) 
            WRITE (NVFIL,900) NAME(1:I)
         ELSE
            WRITE (NVFIL,905) NAME(1:I)
         ENDIF
         NVERR = NVERR + 1
      ENDIF
C
C --- A SECTION ENDS IN EITHER TWO CONSECUTIVE BLANK LINES OR A NEW
C ---  HEADER (NON-BLANK COLUMN 1 )
C
      CALL GETLIN
C
C --- EOF IS LINE  "   +----------------------------------------------+"
C
30    IF (EOF) RETURN      
      IF (LINE(1:1) .EQ. ' ') THEN
         IF (LINE(ILAB:ILAB+6) .EQ. ' ') THEN  
            CALL GETLIN                        ! CONTINUATION
            GO TO 30
         ELSE IF (LINE .NE. ' ') THEN
            GO TO 20                           ! NEW LABEL
         ELSE
            CALL GETLIN                        ! BLANK LINE
            IF ((LINE .NE. ' ') .AND. (LINE (1:1) .EQ. ' ')) GO TO 20
         ENDIF
      ENDIF
      RETURN
900   FORMAT('--- Error, Label ',A,' is not referenced.')
905   FORMAT('--- Error, FORMAT statement ',A,' is not referenced.')
      END
C
C---END DOLAB
C
      SUBROUTINE MERGE
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          MERGE            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          MERGE               
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          MERGE THE VARIABLES FROM THE PRESENT SUBPROGRAM IN WITH
C*          THE REST OF THE VARIABLES.
C*
C*     COMMON BLOCKS :
C*          FILES, VCOM, ONESUB, SUBPRS
C*
C*     FILE REFERENCES :
C*          NVFIL
C*
C*     SUBPROGRAM REFERENCES :
C*          SORTR
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXSUB=400,MAXVAR=3000,MAXVPS=1000,MREF=225,MXCHR=12,
     $           MXCSUB=60, MAXCOM=150)
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / ONESUB / SNAMES(MAXVPS), STYPE(MAXVPS), SIZE(MAXVPS),
     $                  VCOMM(MAXVPS), SCOMMN(0:MXCSUB), NSVAR
      COMMON / VCOM / NUMVAR, VARNAM(MAXVAR), VARTYP(MAXVAR),
     $                VARSIZ(MAXVAR), VCOMMN(MAXVAR), XREF(MAXVAR)
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      CHARACTER *(MXCHR) VARNAM, SUBNAM, VCOMMN, VCOMM, SCOMMN, SNAMES,
     $              TNAME(MAXVAR), TVCOM(MAXVAR), SNAME, VNAME
      CHARACTER *100 CHART
      CHARACTER *10 REFS
      CHARACTER *3 PRSECT
      CHARACTER *1 TYPE, CFLAG, VARTYP, TVTYPE(MAXVAR), STYPE
      CHARACTER *(MAXSUB) XREF, TXREF(MAXVAR)
      INTEGER VARSIZ, TSIZE(MAXVAR), SIZE
      DIMENSION INDX(MAXVPS)
C
      IF ( NSVAR .EQ. 0 ) RETURN
C
C --- COPY EXISTING STUFF TO TEMP STORAGE
C
      DO 10 I = 1, NUMVAR
         TNAME(I) = VARNAM(I)
         TXREF(I) = XREF(I)
         TVCOM(I) = VCOMMN(I)
         TVTYPE(I)= VARTYP(I)
         TSIZE(I) = VARSIZ(I)
10    CONTINUE
C
C --- SORT NEW STUFF ( ARRAYS AND VARIABLES NEED TO BE SORTED )
C
      CALL SORTR ( SNAMES, NSVAR, INDX )
C
C --- NOW MERGE NEW STUFF AND OLD STUFF
C
C --- IF NO OLD STUFF, JUST COPY NEW STUFF
C
      NPOS = NUMSUB - 1
      IF (NUMVAR .EQ. 0) THEN
         DO 15 I = 1, NSVAR
            VARNAM(I) = SNAMES(I)
            XREF(I)   = ' '
            XREF(I)(NPOS:NPOS) = 'X'
            VARTYP(I) = STYPE(INDX(I))
            VARSIZ(I) = SIZE(INDX(I))
            VCOMMN(I) = VCOMM(INDX(I))
15       CONTINUE
         NUMVAR = NSVAR
         RETURN
      ENDIF
C
C --- IV = NEXT ENTRY IN OLD STUFF
C --- IS = NEXT ENTRY IN NEW STUFF
C --- IT = NEXT ENTRY IN OUTPUT ARRAY
C
      IV = 1
      IS = 1
      IT = 0
C
C --- NEXT VARIABLE IN ORIGINAL LIST, COPY IT
C
20    IF ( TNAME(IV) .LT. SNAMES(IS)) THEN
         IT = IT + 1
         IF (IT .GT. MAXVAR) CALL FATAL
     $      ('NUMBER OF VARIABLES EXCEEDS DIMENSION OF PROGRAM',MAXVAR)
         VARNAM(IT) = TNAME(IV)
         XREF(IT)   = TXREF(IV)
         VARTYP(IT) = TVTYPE(IV)
         VARSIZ(IT) = TSIZE(IV)
         VCOMMN(IT) = TVCOM(IV)
         IV = IV + 1
C
C --- NEXT VARIABLE IN SUBPROGRAM LIST, COPY IT / CREATE XREF ENTRY
C
      ELSEIF ( TNAME(IV) .GT. SNAMES(IS)) THEN
         IT = IT + 1
         IF (IT .GT. MAXVAR) CALL FATAL
     $      ('NUMBER OF VARIABLES EXCEEDS DIMENSION OF PROGRAM',MAXVAR)
         VARNAM(IT) = SNAMES(IS)
         XREF(IT)   = ' '
         XREF(IT)(NPOS:NPOS) = 'X'
         VARTYP(IT) = STYPE(INDX(IS))
         VARSIZ(IT) = SIZE(INDX(IS))
         VCOMMN(IT) = VCOMM(INDX(IS))
         IS = IS + 1
C
C --- THEY ARE THE SAME, MERGE THEM
C
      ELSE
         IT = IT + 1
         IF (IT .GT. MAXVAR) CALL FATAL
     $      ('NUMBER OF VARIABLES EXCEEDS DIMENSION OF PROGRAM',MAXVAR)
         VARNAM(IT) = SNAMES(IS)
         XREF(IT)   = TXREF(IV)
         XREF(IT)(NPOS:NPOS) = 'X'
         VARTYP(IT) = TVTYPE(IV)
         VARSIZ(IT) = TSIZE(IV)
         VCOMMN(IT) = TVCOM(IV)
C
C ----- COMPARE OLD AND NEW DEFINITIONS FOR COMPATABILITY
C
         IF (VARTYP(IT) .NE. STYPE(INDX(IS))) THEN
                 WRITE(NVFIL,900)VARNAM(IT)(1:LENGTH(VARNAM(IT)))
                 WRITE(NVFIL,905)VARTYP(IT),STYPE(INDX(IS))
                 NVERR = NVERR + 1
905      FORMAT(' OLD TYPE ',A,T40,'NEW TYPE ',A)
         ENDIF
         IF (VARSIZ(IT) .NE. SIZE(INDX(IS))) THEN
                 WRITE(NVFIL,910)VARNAM(IT)(1:LENGTH(VARNAM(IT)))
                 WRITE(NVFIL,915)VARSIZ(IT),SIZE(INDX(IS))
                 NVERR = NVERR + 1
915      FORMAT(' OLD SIZE ',I6,T40,'NEW SIZE ',I6)
         ENDIF
         IF((VCOMMN(IT) .NE. VCOMM(INDX(IS))) .AND.
     $      (VCOMMN(IT)(1:1) .NE. '$')) THEN
                 WRITE(NVFIL,920)VARNAM(IT)(1:LENGTH(VARNAM(IT)))
                 WRITE(NVFIL,925) VCOMMN(IT), VCOMM(INDX(IS))
925      FORMAT(' OLD COMMON ',A,T40,'NEW COMMON ',A)
         ENDIF
         IS = IS + 1
         IV = IV + 1
      ENDIF
C
C --- UNTIL ONE ARRAY IS EXHAUSTED
C
      IF ((IS .LE. NSVAR) .AND. (IV .LE. NUMVAR)) GO TO 20
C
C --- COPY THE REST OF THE REMAINING ARRAY
C
      IF (IS .GT. NSVAR) THEN
         DO 30 I = IV, NUMVAR
            IT = IT + 1
            IF (IT .GT. MAXVAR) CALL FATAL
     $      ('NUMBER OF VARIABLES EXCEEDS DIMENSION OF PROGRAM',MAXVAR)
            VARNAM(IT) = TNAME(I)
            XREF(IT)   = TXREF(I)
            VARTYP(IT) = TVTYPE(I)
            VARSIZ(IT) = TSIZE(I)
            VCOMMN(IT) = TVCOM(I)
30          CONTINUE
      ELSE
         DO 40 I = IS, NSVAR
            IT = IT + 1
            IF (IT .GT. MAXVAR) CALL FATAL
     $      ('NUMBER OF VARIABLES EXCEEDS DIMENSION OF PROGRAM',MAXVAR)
            VARNAM(IT) = SNAMES(I)
            XREF(IT)   = ' '
            XREF(IT)(NPOS:NPOS) = 'X'
            VARTYP(IT) = STYPE(INDX(I))
            VARSIZ(IT) = SIZE(INDX(I))
            VCOMMN(IT) = VCOMM(INDX(I))
40          CONTINUE
      ENDIF
      NUMVAR = IT
      RETURN
900   FORMAT('--- Error, type of variable "',A,'" conflicts ',
     $ 'with previous use.')
910   FORMAT('--- Error, size of variable "',A,'" conflicts ',
     $ 'with previous use.')
920   FORMAT('--- Warning, variable "',A,'" previously appeared in a ',
     $ 'different storage unit.')
      END
C                                   
C---END MERGE
C
      SUBROUTINE SUMMAR
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          SUMMAR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          SUMMARIZE                               
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          PRODUCE THE SUMMARY PRINTOUT.                               
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FILES, SUBPRS, COMMNS, VERCOM, FLIST
C*
C*     FILE REFERENCES :
C*          NSRCD, NSUM, NOUT
C*
C*     SUBPROGRAM REFERENCES :
C*          DATE, TIME, CENTER
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          DATE, TIME, AND CENTER ARE IN 'MERLIB'
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          18-NOV-87    CARRIAGECONTROL = 'LIST' ADDED
C*          08-OCT-86    MODIFIED FOR POSTBUG
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXSUB=400,MAXCOM=150,MREF=225,MXCHR=12)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / COMMNS / NAMES(MAXCOM), LENS(MAXCOM), ILENS(MAXCOM), 
     $                  NCERR, NUMCOM, NCOMMN
      COMMON / VERCOM / CNAMES(MAXCOM), NUMS(MAXCOM), NTCOM, NVFERR
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      COMMON / FLIST / LINE, EOF, FF
      CHARACTER *(MXCHR) SUBNAM, SNAME, VNAME, NAMES, CNAMES
      CHARACTER *150 OPSTRG      
      CHARACTER *132 LINE
      CHARACTER *100 CHART
      CHARACTER *97 DIREC
      CHARACTER *15 SAVNAM
      CHARACTER *10 REFS
      CHARACTER *9 ADATE
      CHARACTER *8 ATIME
      CHARACTER *3 PRSECT
      CHARACTER *1 TYPE, CFLAG, FF
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT,
     $        SAVE, RESTOR, Q1Q, EOF, UCHECK, NMLST
C
      OPEN (UNIT=NSUM, STATUS='NEW', CARRIAGECONTROL='LIST',ERR=1000)
      WRITE (NSUM, 910) FF
      CALL DATE ( ADATE )
      CALL TIME ( ATIME )
      DIREC = ' '
      DO 5 I=1,4
         READ(NSRCD,910,END=10,ERR=10) DIREC
5        CONTINUE
      CALL CENTER ( DIREC )
10    WRITE ( NSUM, 900 ) DIREC, ATIME(1:5), ADATE
      NBREAK1 = 75
15    NBREAK1 = NBREAK1 - 1
      IF (OPSTRG(NBREAK1:NBREAK1) .NE. ',') GO TO 15
      NBREAK2 = 150
17    NBREAK2 = NBREAK2 - 1
      IF (OPSTRG(NBREAK2:NBREAK2) .NE. ',') GO TO 17
      WRITE (NSUM,905) OPSTRG(1:NBREAK1), OPSTRG(NBREAK1+1:NBREAK2),
     $                 OPSTRG(NBREAK2+1:150)
      IF ((.NOT. RUNVFY) .AND. (.NOT. RUNVBL) .AND. (.NOT. RUNC)) THEN
         WRITE (NSUM,945)
      ELSE
         NUMSUB = NUMSUB-1
         WRITE(NSUM,950) NUMSUB       
      ENDIF
      IF ( RUNC ) WRITE(NSUM,955)NCERR 
      IF ( RUNVBL ) WRITE(NSUM,956)NVERR
      IF ( RUNVFY ) WRITE(NSUM,957)NVFERR
      WRITE(NSUM,958)
      IF ( RUNC .AND. (NUMCOM .GT. 0) ) THEN
         WRITE(NOUT,960) FF
         DO 20 I = 1, NUMCOM
            WRITE(NOUT,980)NAMES(I),LENS(I)
20       CONTINUE
      ENDIF
      IF ( RUNVBL ) THEN
         WRITE ( NSUM, 920 )
         WRITE ( NSUM, 930 )
         WRITE ( NSUM, 940 )
      ENDIF
      WRITE (NSUM,910) FF
      CLOSE (UNIT=NSUM)
      RETURN
1000  WRITE(NOUT,999)
      RETURN
900   FORMAT(////,100('*'),/,'*',T100,'*',/,
     $ '*           ------ POSTBUG  ',T54,
     $ 'VERSION III.15 (05 - OCT - 1992) ------',T100,'*',/,
     $ 2('*',T100,'*',/),
     $ '*',A98,'*',/,
     $ '*                  ------       STATUS OF DATA SET AS OF ',
     $ A5,' ',A9,T81,'------             *',/,
     $ '*',T100,'*',/,'*',T100,'*',/,'*',T100,'*')
905   FORMAT (                                    
     $ '*         OPTIONS = ',A,T100,'*',/,'*',T20,A,T100,'*',
     $ /,'*',T20,A,T100,'*',/,2('*',T100,'*',/),
     $ 100('*'))
910   FORMAT(A)
920   FORMAT(/////,
     $ T30,'------ Interpretation of VARIABLE Output ------',//,
     $ 'VARIABLE checking produces several normal messages and ',
     $    'potentially several internal errors.  The internal',/,
     $ 'errors are preceded by five asterisks and indicate a ',
     $    'failure with the VARIABLE checking itself.  Normally',/,
     $ 'this is the exceeding of dimensioned limits.  This error ',
     $    'must be corrected by redimensioning the source code of',/,
     $ 'the VARIABLE checking and rerunning the case.',/,
     $ 'Messages preceded by one or two hyphens are errors which the',
     $   ' VARIABLE checking perceives to exist in the target program.',
     $ ///,
     $ 'The message :   ...IS REFERENCED ONLY ONCE   indicates that ',
     $    'the variable listed exists only once in the subprogram',/,
     $ 'and may normally be safely deleted.',//,
     $ 'The message :   THE FIRST REFERENCE TO ... IS NOT AN ',
     $    'INITIALIZATION.   means that the first reference is not',/,
     $ 'an assignment, data, or SUBROUTINE statement.  ',
     $    'This may not actually be an error, but careful scrutiny of'/
     $ 'those variables that produce this error often indicates an ',
     $    'error in the logic of the subprogram.',/)
930   FORMAT (
     $ 'The message :   ...APPEARS TO BE UNUSED.   is always an error',
     $    ' unless one of the references is a WRITE statement or',/,
     $ 'the variable is EQUIVALENCEd to a variable that is used.  ',
     $    'It refers to the fact that the variable may be ',/,
     $ 'calculated but is never passed to another ',
     $    'routine or used in a calculation.',//,
     $ 'The message :   ...APPEARS TO BE UNINITIALIZED.  is very ',
     $    'similar to the above message except that the variable',/,
     $ 'is used in a calculation but does not appear to be assigned ',
     $    'a value.  Again, a READ statement or EQUIVALENCE may',/,
     $ 'result in this message being produced erroneously.',//,
     $ 'The message :   LABEL ... IS ONLY REFERENCED ONCE. indicates ',
     $    'that the label can be deleted.')
940   FORMAT ( /,
     $ 'The message :   VARIABLE ... PREVIOUSLY APPEARED IN A ',
     $    'DIFFERENT STORAGE UNIT. means that the variable',/,
     $ 'is in a COMMON block or in local storage, but was in a ',
     $    'different COMMON block or routine previously.',//,
     $ 'The message :   SIZE (TYPE) OF VARIABLE ... CONFLICTS ',
     $    'WITH PREVIOUS USAGE. means that the variable has been ',/,
     $ 'used before, but in the previous usage the declared size',
     $    '(type) was different than the declared size(type) in the',/,
     $ 'present routine.  Size = 0 implies a character variable.')
945   FORMAT
     $('*',T100,'*',/,
     $ '*',T27,'------  NO OPTIONS CHOSEN - ',
     $ 'NO OUTPUT PRODUCED ',
     $ '------',T100,'*',/,'*',T100,'*')
950   FORMAT
     $('*',T100,'*',/,
     $ '*',T25,'------  NUMBER OF SUBPROGRAMS CHECKED = ',I3,2X,
     $ '------',T100,'*',/,'*',T100,'*',/,100('*'))
955   FORMAT
     $('*',T100,'*',/,
     $ '*',T25,'------  NUMBER OF COMMON ERRORS FOUND   = ',I3,2X,
     $ '------',T100,'*',/,'*',T100,'*')
956   FORMAT
     $('*',T100,'*',/,
     $ '*',T25,'------  NUMBER OF VARIABLE ERRORS FOUND = ',I3,2X,
     $ '------',T100,'*',/,'*',T100,'*')
957   FORMAT
     $ ('*',T100,'*',/,
     $ '*',T25,'------  NUMBER OF UNUSED COMMONS FOUND  = ',I3,2X,
     $ '------',T100,'*',/,'*',T100,'*')
958   FORMAT(100('*'))
960   FORMAT (A,///,10X,'COMMON NAME                     SIZE',/,
     $                10X,'------ ----                    ------',//)
980   FORMAT (12X,A12,17X,I7)
999   FORMAT (' ***** ERROR, UNABLE TO OPEN SUMMARY FILE - ',
     $        'NO SUMMARY WILL BE PRODUCED.')
      END
C
C---END SUMMAR
C            
      SUBROUTINE UVCHK 
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           UVCHK           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          UNIQUE VARIABLE CHECK
C*
C*     AUTHOR :
C*          L JURGELEIT
C*          MS 219-3
C*          AMES RESEARCH CENTER
C*          MOFFETT FIELD, CA  94035
C*          (415) 694-5578
C*
C*     PURPOSE :
C*          CHECK ALL VARIABLES FOR UNIQUENESS WITHIN FIRST
C*          SIX CHARACTERS
C*
C*     COMMON BLOCKS :
C*          FILES, VCOM, FLIST
C*
C*     SUBPROGRAM REFERENCES :
C*          LENGTH
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          12-JAN-1988  -  INITIAL VERSION
C*
C***********************************************************************
C*                                                                
      PARAMETER (MAXVAR=3000,MAXSUB=400,NUCHAR=6,MXCHR=12)
      COMMON / FLIST / LINE, EOF, FF
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / VCOM / NUMVAR, VARNAM(MAXVAR), VARTYP(MAXVAR),
     $                VARSIZ(MAXVAR), VCOMMN(MAXVAR), XREF(MAXVAR)
      CHARACTER *(MAXSUB) XREF
      CHARACTER *(MXCHR) VARNAM, VCOMMN, VTEMPA(MAXVAR), T1, T2,
     $ NOTU(MAXVAR)
      CHARACTER *132 LINE
      CHARACTER *1 VARTYP, FF
      INTEGER VARSIZ
      LOGICAL WRTLST, EOF
C
      WRITE (NVFIL,905) FF
      WRITE (NVFIL,900)
      NUMSIX = 0
C
C --- CHECK ALL VARIABLES FOR LENGTH >= 6
C
      DO 10 I = 1, NUMVAR
         IF (LENGTH(VARNAM(I)) .GE. NUCHAR) THEN
            NUMSIX = NUMSIX + 1
            VTEMPA(NUMSIX) = VARNAM(I)
         ENDIF
10    CONTINUE
      IF (NUMSIX .GT. 1) THEN
C
C --- CHECK THE VARIABLES
C
         WRTLST = .FALSE.
         T1 = VTEMPA(1)
         T2 = VTEMPA(2)
         NNU = 0
         I   = 2
20       IF (T1(1:NUCHAR) .EQ. T2(1:NUCHAR)) THEN
C
C --- VARIABLE NOT UNIQUE
C
            NNU = NNU + 1
            NOTU(NNU) = T1
            IF (.NOT. WRTLST) WRTLST = .TRUE.
         ELSE IF (WRTLST) THEN
            NNU = NNU + 1
            NOTU(NNU) = T1
            WRTLST  = .FALSE.
         ENDIF
         I = I + 1
         IF (I .GT. NUMSIX) GO TO 30
         T1 = T2
         T2 = VTEMPA(I)
         GO TO 20
30       IF (NNU .GT. 0) THEN
C
C --- WRITE THE VARIABLES THAT AREN'T UNIQUE
C
            WRITE (NVFIL,910)
            DO 40 I = 1,NNU
               WRITE(NVFIL,915) NOTU(I)
40          CONTINUE
         ELSE
C
C --- ALL VARIABLES UNIQUE WITHIN FIRST 6 CHAR.
C
            WRITE(NVFIL,940) NUCHAR
         ENDIF      
      ELSE
C
C --- ONE OR NO VARIABLES WITH LENGTH >= 6
C
         IF (NUMSIX .EQ. 1) THEN
            WRITE(NVFIL,925) NUCHAR
         ELSE
            WRITE(NVFIL,920) NUCHAR
         ENDIF
      ENDIF
      WRITE (NVFIL,930)
      RETURN  
900   FORMAT(//'********** VARIABLE UNIQUENESS CHECKING **********')
905   FORMAT(A)
910   FORMAT(/,'The following variables ARE NOT UNIQUE: ',/)
915   FORMAT(10X,A)
920   FORMAT(/'All variables have length < ',I1,' characters.')
925   FORMAT(/'Only one variable has length >= ',I1,' characters.')
930   FORMAT(/'********** END UNIQUENESS CHECKING ***************')
940   FORMAT(/'All variables are unique within the first ',I1,
     $         ' characters.')
      END
C
C---END UVCHK
C
      SUBROUTINE CHARTR 
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          CHARTR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          CHARTER              
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF 94035              
C*          (415)965-5578                           
C*
C*     PURPOSE :
C*          PRINT OUT THE COMMON BLOCK / SUBPROGRAM CROSS-REFERENCE CHART
C*
C*     INTERNAL WORK AREAS :
C*          BLOCK - CONTAINS THE TEXT FOR THE UPPER,LEFT-HAND CORNER
C*          CROW  - ONE ROW OF THE CHART, WHILE IT IS BEING BUILT
C*
C*     COMMON BLOCKS :
C*          RUNOPT, FILES, COMMNS, SUBPRS, FLIST
C*
C*     FILE REFERENCES :
C*          NOUT - PRINT OUT CROSS-REFERENCE CHART
C*
C*     SUBPROGRAM REFERENCES :
C*          SORTR
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          PRINTOUT ASSUMES 133 COLUMN OUTPUT
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          05-JUN-84     SORT OPTION ADDED
C*          30-JAN-84     INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXSUB=400,MAXCOM=150,MXCHR=12)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / COMMNS / NAMES(MAXCOM), LENS(MAXCOM), ILENS(MAXCOM), 
     $                  NCERR, NUMCOM, NCOMMN
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / FLIST / LINE, EOF, FF
      DIMENSION INDX(300)
      CHARACTER *(MXCHR) SUBNAM, SNAME, NAMES
      CHARACTER *150 OPSTRG
      CHARACTER *132 LINE 
      CHARACTER *120 CROW
      CHARACTER *100 CHART
      CHARACTER *15 SAVNAM
      CHARACTER *11 BLOCK(10)
      CHARACTER *1 FF
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT,
     $        SAVE, RESTOR, Q1Q, EOF, UCHECK, NMLST
      DATA BLOCK/ '  COMMONS :',
     $            ' \        :',
     $            '   \      :',
     $            '     \    :',
     $            '       \  :',
     $            'SUB-     \:',
     $            'PROGRAMS  :',
     $            '          :',
     $            '          :',
     $            '          :'/
C
      IF ( SORT ) THEN
         CALL SORTR ( SUBNAM, NUMSUB, INDX )
      ELSE
         DO 5 I = 1, NUMSUB
            INDX(I) = I
5        CONTINUE
      ENDIF
      ISTART = 1
      IF ( NUMCOM .GT. 32 ) THEN
         NEND = 32
      ELSE
         NEND = NUMCOM
      ENDIF
C
C --- PRINT UP TO 32 COMMONS PER PAGE
C
C --- HEADER FOR CROSS-REFERENCE MAP
C
10    WRITE(NOUT,940) FF
      DO 20 I = 1,10
         WRITE(NOUT,900)BLOCK(I),(NAMES(J)(I:I),J=ISTART,NEND)
20    CONTINUE
      WRITE(NOUT,910)
      DO 30 I = 1, NUMSUB
         CROW = ' '
         IPTR = 1
         DO 25 II = ISTART, NEND
            CROW(IPTR:IPTR+2) = '  '//CHART(INDX(I))(II:II)
            IPTR = IPTR + 3
            IF ( MOD(II,4) .EQ. 0 ) THEN
               IF ( IPTR .LT. 110 ) THEN
                  CROW(IPTR:IPTR+2) = '  .'
                  IPTR = IPTR + 3
               ENDIF
            ENDIF
25       CONTINUE
         CROW(120:120) = '.'
         WRITE(NOUT,920)SUBNAM(I),CROW
         IF (MOD(I,3) .EQ. 0) WRITE(NOUT,925)
30       CONTINUE
      ISTART = NEND + 1
      NEND   = ISTART + 31
      IF ( NEND .GT. NUMCOM ) NEND = NUMCOM
      IF ( NEND .GE. ISTART ) GO TO 10
      RETURN
900   FORMAT (A11,8(4(2X,A1),'  .'))
910   FORMAT (132('-'))
920   FORMAT (A10,':',A120)
925   FORMAT (10X,':',8(14X,'.'))
940   FORMAT (A,//)
      END
C
C---END CHARTR
C
      SUBROUTINE SORTR ( ARRAY, NUM, INDX )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **           SORTR           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          SORT ARRAY
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF 94035              
C*          (415)965-5578                           
C*
C*     PURPOSE :
C*          PRODUCE A SORTED INDEX POINTER ARRAY
C*
C*     METHODOLOGY :
C*          SHELLSORT                        
C*
C*     INPUT ARGUMENTS :
C*          NUM    - NUMBER OF ELEMENTS IN ARRAY
C*          INDX   - INDEX ARRAY
C*          ARRAY  - ARRAY TO BE SORTED
C*
C*     ASSUMPTIONS AND RESTRICTIONS :
C*          THE TYPE OF VARIABLES ARRAY AND TEMPA ARE UNIQUE FOR
C*           EACH APPLICATION.
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          06/05/84     INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MXCHR=12)
      DIMENSION ARRAY(1), INDX(1)
      CHARACTER *(*) ARRAY
      CHARACTER *(MXCHR) TEMPA
      INTEGER TEMPI
      LOGICAL DONE
C
      DO 10 I = 1, NUM
         INDX(I) = I
10       CONTINUE
      IF (NUM .LE. 1) RETURN
      JUMP = NUM
20    JUMP = JUMP / 2
30    DONE = .TRUE.
      NJ   = NUM-JUMP
      DO 40 J = 1, NJ
         I = J + JUMP
         IF (ARRAY(J) .GT. ARRAY(I))THEN
            DONE     = .FALSE.
            TEMPA    = ARRAY(J)
            ARRAY(J) = ARRAY(I)
            ARRAY(I) = TEMPA
            TEMPI    = INDX(J)
            INDX(J)  = INDX(I)
            INDX(I)  = TEMPI
         ENDIF
40       CONTINUE
      IF (.NOT. DONE) GO TO 30
      IF (JUMP .GT. 1) GO TO 20
      RETURN
      END
C
C---END SORT
C
C
      SUBROUTINE CHARTV
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          CHARTV           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          CROSS-REFERENCE CHART                   
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          PRODUCE THE VARIABLE/SUBPROGRAM CROSS-REFERENCE CHART.      
C*
C*     COMMON BLOCKS :
C*          FILES, VCOM, SUBPRS, FLIST
C*
C*     FILE REFERENCES :
C*          NVFIL
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXSUB=400,MAXVAR=3000,MXCHR=12)
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / VCOM / NUMVAR, VARNAM(MAXVAR), VARTYP(MAXVAR),
     $                VARSIZ(MAXVAR), VCOMMN(MAXVAR), XREF(MAXVAR)
      COMMON / SUBPRS / SUBNAM(MAXSUB), CHART(MAXSUB), SNAME, NUMSUB
      COMMON / FLIST / LINE, EOF, FF
      CHARACTER *(MXCHR) VARNAM, SUBNAM, VCOMMN, SNAME
      CHARACTER *132 LINE 
      CHARACTER *120 CROW
      CHARACTER *100 CHART
      CHARACTER *11 BLOCK(10)
      CHARACTER *1 VARTYP, FF
      CHARACTER *(MAXSUB) XREF
      INTEGER VARSIZ
      LOGICAL EOF
      DATA BLOCK/ ' SUBPROGS :',
     $            ' \        :',
     $            '   \      :',
     $            '     \    :',
     $            '       \  :',
     $            '         \:',
     $            'VARIABLES :',
     $            '          :',
     $            '          :',
     $            '          :'/
C
      ISTART = 1
      IF ( NUMSUB .GT. 32 ) THEN
         NEND = 32
      ELSE
         NEND = NUMSUB
      ENDIF
C
C --- HEADER FOR CROSS-REFERENCE MAP (ONCE FOR EACH PAGE REQUIRED)
C
C --- A PAGE MAY CONTAIN UP TO 32 SUBPROGRAMS...
C ---  'PAGE' REFERES TO A VERTICAL SLICE OF THE CHART AND MAY BE MORE
C ---  THAN ONE SHEET OF PAPER ON THE PRINTER
C
10    WRITE(NVFIL,940) FF
      DO 20 I = 1,10
         WRITE(NVFIL,900)BLOCK(I),(SUBNAM(J)(I:I),J=ISTART,NEND)
20       CONTINUE
      WRITE(NVFIL,910)
C
C --- LOOP OVER ALL VARIABLES
C
      DO 30 I = 1, NUMVAR
         CROW = ' '
         IPTR = 1
C
C ------ BUILD ONE OUTPUT ROW FOR THIS PAGE OF THE CHART
C
         DO 25 II = ISTART, NEND
            CROW(IPTR:IPTR+2) = '  '//XREF(I)(II:II)
            IPTR = IPTR + 3
            IF ( MOD(II,4) .EQ. 0 ) THEN
               IF ( IPTR .LT. 110 ) THEN
                  CROW(IPTR:IPTR+2) = '  .'
                  IPTR = IPTR + 3
               ENDIF
            ENDIF
25       CONTINUE
C
         CROW(120:120) = '.'
         WRITE(NVFIL,920)VARNAM(I),CROW
         IF (MOD(I,3) .EQ. 0) WRITE(NVFIL,925)
30       CONTINUE
C
C --- IF MORE PAGES ARE REQUIRED, RESET POINTERS FOR NEXT PAGE
C
      ISTART = NEND + 1
      NEND   = ISTART + 31
      IF ( NEND .GT. NUMSUB ) NEND = NUMSUB
      IF ( NEND .GE. ISTART ) GO TO 10
      RETURN
900   FORMAT (A11,8(4(2X,A1),'  .'))
910   FORMAT (132('-'))
920   FORMAT (A10,':',A120)
925   FORMAT (10X,':',8(14X,'.'))
940   FORMAT (A,//)
      END
C
C---END CHART
C
      SUBROUTINE DICOUT
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          DICOUT           **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          DICTIONARY OUTPUT                       
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA  94035                
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          PRINT OUT THE PARTIAL DATA DICTIONARY.                      
C*
C*     COMMON BLOCKS :
C*          FILES, VCOM, FLIST
C*
C*     FILE REFERENCES :
C*          NDICT
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          18-NOV-87    CARRIAGECONTROL='LIST' ADDED
C*           7-NOV-85    INITIAL VERSION
C*
C***********************************************************************
C*
      PARAMETER (MAXSUB=400,MAXVAR=3000,MXCHR=12)
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / VCOM / NUMVAR, VARNAM(MAXVAR), VARTYP(MAXVAR),
     $                VARSIZ(MAXVAR), VCOMMN(MAXVAR), XREF(MAXVAR)
      COMMON / FLIST / LINE, EOF, FF
      CHARACTER *(MXCHR) VARNAM, VCOMMN
      CHARACTER *132 LINE
      CHARACTER *5 VERSION
      CHARACTER *1 VARTYP, FF
      CHARACTER *(MAXSUB) XREF
      LOGICAL EOF
      INTEGER VARSIZ
C
      OPEN (UNIT=NDICT, STATUS='NEW', CARRIAGECONTROL='LIST', ERR=1000)
      CALL GETFVERS (NDICT, VERSION)
      CALL LIB$SET_SYMBOL ('BUG_DV',VERSION)
      WRITE(NDICT,910) FF
      DO 10 I=1,NUMVAR
         IF(MOD(I,50) .EQ. 0) WRITE(NDICT,910) FF
         WRITE(NDICT,920) VARNAM(I), VARTYP(I), VARSIZ(I), VCOMMN(I)
10       CONTINUE
      CLOSE (UNIT=NDICT)
      RETURN
1000  WRITE (NOUT,999)
      RETURN
910   FORMAT(A,T25,' Data Dictionary.'//
     $ 'Variable',T13,'Type',T20,'Size',T27,'Storage Unit')
920   FORMAT(A,T14,A,T18,I8,T30,A)
999   FORMAT(' ****** ERROR OPENING DICTIONARY FILE - ',
     $       'NO DICTIONARY PRODUCED.')
      END
C
C--- END DICOUT
C
      SUBROUTINE MESSG
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          MESSG            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          MESSAGE
C*
C*     AUTHOR :
C*          LAURA JURGELEIT
C*          MS219-3                                 
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CA.   94035              
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          OUTPUT NAME OF SUBPROGRAM AND COMMON BLOCKS NOT USED
C*          
C*     COMMON BLOCKS :
C*          FILES, FLAGS, VERCOM, ONEVAR, RUNOPT, COMMNS
C*
C*     FILE REFERENCES :
C*          NOUT
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          AUGUST 26, 1986         INITIAL VERSION 
C*           
C*
C***********************************************************************
C*
      PARAMETER (MAXCOM=150,MREF=225,MXCHR=12)
      COMMON / RUNOPT / RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT,
     $                  SORT, SAVE, RESTOR, SAVNAM, OPSTRG, NPRINT, Q1Q,
     $                  UCHECK, NMLST
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      COMMON / FLAGS / COM, DONE, GO, DID, NML, BD
      COMMON / VERCOM / CNAMES(MAXCOM), NUMS(MAXCOM), NTCOM, NVFERR
      COMMON / ONEVAR / PRSECT, TYPE, ISIZE, VNAME, CFLAG, REFS(MREF), 
     $                  NREF, NVERR, NCOM, NREFS(MREF), NSECT,
     $                  IDLEN(MAXCOM)
      COMMON / COMMNS / NAMES(MAXCOM), LENS(MAXCOM), ILENS(MAXCOM), 
     $                  NCERR, NUMCOM, NCOMMN
      CHARACTER *(MXCHR) CNAMES, NAMES, VNAME
      CHARACTER *150 OPSTRG
      CHARACTER *15 SAVNAM
      CHARACTER *10 REFS
      CHARACTER *3 PRSECT
      CHARACTER *1 CFLAG, TYPE
      LOGICAL RUNVFY, RUNVBL, RUNC, MAPV, MAPC, LFIRST, DICT, SORT, 
     $        SAVE, RESTOR, Q1Q, UCHECK, NMLST
      LOGICAL COM, DONE, GO, DID, NML, BD
C
      IF ( .NOT. RUNVFY ) THEN
         DO 10 I = 1, NCOM
            IF (NPRINT .GT. 0) WRITE(NOUT,900) CNAMES(I)
            IF (ILENS(I) .NE. IDLEN(I)) THEN
               LENS(I) = ILENS(I)
               WRITE(NOUT,920)CNAMES(I),ILENS(I),IDLEN(I)
            ENDIF
10       CONTINUE
         GO TO 30
      ENDIF
      DO 20 I = 1,NCOM
         IF (ILENS(I) .NE. IDLEN(I)) THEN
            WRITE(NOUT,920)CNAMES(I),ILENS(I),IDLEN(I)
         ENDIF
         IF (DID .AND. (.NOT. DONE) .AND. COM .AND. GO) THEN
            IF (NUMS(I) .NE. -1) THEN
C
C --- UNUSED COMMON BLOCK
C
               WRITE(NOUT,910) CNAMES(I)
               NVFERR = NVFERR + 1
            ELSE IF (RUNC .AND. (NPRINT .GT. 0)) THEN
               WRITE(NOUT,900) CNAMES(I)
            ENDIF
         ELSE IF (RUNC .AND. (NPRINT .GT. 0)) THEN
            WRITE(NOUT,900) CNAMES(I)
         ENDIF
20    CONTINUE
30    DO 40 J = 1,MAXCOM
         ILENS(J) = 0
         IDLEN(J) = 0
40    CONTINUE
      RETURN
900   FORMAT('      ',A)
910   FORMAT('      ',A,' - NOT REFERENCED')
920   FORMAT
     $   (/'******************************************************',/,
     $    '*                                                    *',/,
     $    '*  ERROR... COMMON BLOCK LENGTH DISAGREEMENT...      *',/,
     $    '*                                                    *',/,
     $    '*  ',A12,'DEFINED LENGTH  = ',I7,'             *',/,
     $    '*              PREVIOUS LENGTH = ',I7,'             *',/,
     $    '*                                                    *',/,
     $    '******************************************************',/)
      END
C      
C---END MESSG
C                
      SUBROUTINE FATAL ( MESS, LIMIT )
C*
C*                  *******************************
C*                  *******************************
C*                  **                           **
C*                  **          FATAL            **
C*                  **                           **
C*                  *******************************
C*                  *******************************
C*
C*     SUBPROGRAM :
C*          FATAL                                   
C*
C*     AUTHOR :
C*          ART RAGOSTA                             
C*          MS 219-3                                
C*          AMES RESEARCH CENTER                    
C*          MOFFETT FIELD, CALIF    94035           
C*          (415) 694-5578                          
C*
C*     PURPOSE :
C*          PRODUCE A FATAL ERROR MESSAGE.                              
C*
C*     INPUT ARGUMENTS :
C*          MESS - THE TEXT OF THE ERROR MESSAGE
C*          LIMIT - VALUE OF ARRAY DIMENSION EXCEEDED
C*                  (= 0 FOR FILE ERRORS)
C*
C*     COMMON BLOCKS :
C*          FILES
C*
C*     FILE REFERENCES :
C*          NOUT
C*
C*     SUBPROGRAM REFERENCES :
C*          SUMMAR
C*
C*     LANGUAGE AND COMPILER :
C*          ANSI FORTRAN 77
C*
C*     CHANGE HISTORY :
C*          30-JUL-85    INITIAL VERSION
C*
C***********************************************************************
C*
      COMMON / FILES / NIN, NOUT, NSUM, NRSTR, NOPT, NDICT, NVFIL, NSRCD
      CHARACTER *(*) MESS
C
      WRITE (NOUT,900) MESS
      IF (LIMIT .GT. 0) WRITE (NOUT,910) LIMIT
      CALL SUMMAR
      STOP
900   FORMAT(/' ****** Fatal error, ',A)
910   FORMAT( '        The limit on this parameter is ',I4)
      END
C
C---END FATAL
C
